Object subclass: #DigitalResonator
	instanceVariableNames: 'a b c p1 p2 name '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!

!DigitalResonator methodsFor: 'initialization' stamp: 'len 4/8/98 02:23'!
initialize
	p1 _ 0.0.
	p2 _ 0.0! !


!DigitalResonator methodsFor: 'accessing' stamp: 'len 5/4/98 01:49'!
frequency: freq bandwidth: bw
	"Convert formant frequencies and bandwidth into
	resonator difference equation coefficients."
	| arg r |
	arg _ Float pi negated / self samplingRate * bw.
	r _ arg exp.
	c _ r squared negated.
	arg _ Float pi * 2.0 / self samplingRate * freq.
	b _ r * arg cos * 2.0.
	a _ 1.0 - b - c! !

!DigitalResonator methodsFor: 'accessing' stamp: 'len 4/8/98 00:03'!
frequency: freq bandwidth: bw gain: gain
	"Convert formant frequencies and bandwidth into
	resonator difference equation coefficients."
	self frequency: freq bandwidth: bw.
	a _ a * gain! !

!DigitalResonator methodsFor: 'accessing' stamp: 'len 4/7/98 23:46'!
name
	^ name! !

!DigitalResonator methodsFor: 'accessing' stamp: 'len 4/7/98 23:46'!
name: aString
	name _ aString! !

!DigitalResonator methodsFor: 'accessing' stamp: 'len 5/4/98 01:50'!
samplingRate
	^ VoiceSynthesizer samplingRate! !


!DigitalResonator methodsFor: 'operations' stamp: 'len 4/9/98 23:11'!
value: aFloat
	| answer |
	answer _ a * aFloat + (b * p1) + (c * p2).
	p2 _ p1.
	p1 _ answer.
	^ answer! !


!DigitalResonator methodsFor: 'printing' stamp: 'len 4/30/98 22:13'!
printOn: aStream
	self name isNil ifTrue: [^ super printOn: aStream].
	aStream nextPutAll: self name! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DigitalResonator class
	instanceVariableNames: ''!

!DigitalResonator class methodsFor: 'instance creation' stamp: 'len 4/8/98 02:23'!
new
	^ super new initialize! !


DigitalResonator subclass: #DigitalAntiResonator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!

!DigitalAntiResonator methodsFor: 'accessing' stamp: 'len 5/4/98 01:30'!
frequency: freq bandwidth: bw
	"Convert formant frequencies and bandwidth into
	anti-resonator difference equation coefficients."

"	| f |
	f _ freq negated.
	f >= 0 ifTrue: [f _ -1.0]."
	"First compute ordinary resonator coefficients."
	super frequency: freq bandwidth: bw.
	"Now convert to anti-resonator coefficients."
	a _ a reciprocal.
	b _ b * a negated.
	c _ c * a negated! !


!DigitalAntiResonator methodsFor: 'operations' stamp: 'len 4/9/98 22:46'!
value: aNumber
	| answer |
	answer _ a * aNumber + (b * p1) + (c * p2).
	p2 _ p1.
	p1 _ aNumber.
	^ answer! !


Object subclass: #Phoneme
	instanceVariableNames: 'ipa name example features '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!

!Phoneme methodsFor: 'accessing' stamp: 'len 4/9/98 23:26'!
example
	^ example! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/9/98 23:26'!
example: aString
	example _ aString! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/30/98 23:59'!
features
	^ features! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/30/98 23:59'!
features: aCollection
	features _ aCollection! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/10/98 15:37'!
ipa
	^ ipa! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/10/98 15:37'!
ipa: aCharacter
	ipa _ aCharacter! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/9/98 23:13'!
name
	^ name! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/9/98 23:13'!
name: aString
	name _ aString! !


!Phoneme methodsFor: 'printing' stamp: 'len 4/10/98 16:11'!
printOn: aStream
	aStream nextPut: ipa! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Phoneme class
	instanceVariableNames: 'english '!

!Phoneme class methodsFor: 'instance creation' stamp: 'len 4/30/98 23:58'!
ipa: aCharacter name: aString example: anotherString features: anArray
	^ self new ipa: aCharacter; name: aString; example: anotherString; features: anArray! !


!Phoneme class methodsFor: 'class initialization' stamp: 'len 5/1/98 00:02'!
initialize
	"Phoneme initialize."

	english _ Set new: 37.
	english
		add: (self ipa: $p name: 'p' example: '' features: #(vls blb stp));
		add: (self ipa: $t name: 't' example: '' features: #(vls alv stp));
		add: (self ipa: $k name: 'k' example: '' features: #(vls vel stp));
		add: (self ipa: $b name: 'b' example: '' features: #(vcd blb stp));
		add: (self ipa: $d name: 'd' example: '' features: #(vcd alv stp));
		add: (self ipa: $g name: 'g' example: '' features: #(vcd vel stp));
		add: (self ipa: $m name: 'm' example: '' features: #(blb nas));
		add: (self ipa: $n name: 'n' example: '' features: #(alv nas));
		add: (self ipa: $N name: 'N' example: 'siNg' features: #(vel nas));
		add: (self ipa: $f name: 'f' example: '' features: #(vls lbd frc));
		add: (self ipa: $T name: 'T' example: 'THin' features: #(vls dnt frc));
		add: (self ipa: $s name: 's' example: '' features: #(vls alv frc));
		add: (self ipa: $S name: 'S' example: 'SHed' features: #(vls pla frc));
		add: (self ipa: $h name: 'h' example: '' features: #(glt apr));
		add: (self ipa: $v name: 'v' example: '' features: #(vcd lbd frc));
		add: (self ipa: $D name: 'D' example: 'THen' features: #(vcd dnt frc));
		add: (self ipa: $z name: 'z' example: '' features: #(vcd alv frc));
		add: (self ipa: $Z name: 'Z' example: '' features: #(vcd pla frc));
		add: (self ipa: $l name: 'l' example: '' features: #(vcd alv lat));
		add: (self ipa: $r name: 'r' example: '' features: #(alv apr));
		add: (self ipa: $R name: '<rzd>' example: '' features: #(rzd));
		add: (self ipa: $w name: 'w' example: '' features: #(vcd lbv apr));
		add: (self ipa: $j name: 'j' example: '' features: #(vcd pal apr));
		add: (self ipa: $I name: 'I' example: 'bId' features: #(smh fnt unr vwl));
		add: (self ipa: $e name: 'E' example: 'bEd' features: #(lmd fnt unr vwl));
		add: (self ipa: $& name: '&' example: 'bAd' features: #(low fnt unr vwl));
		add: (self ipa: $V name: 'V' example: 'bUd' features: #(lmd bck unr vwl));
		add: (self ipa: $0 name: 'A.' example: 'cOd' features: #(low bck rnd vwl));
		add: (self ipa: $U name: 'U' example: 'gOOd' features: #(smh bck rnd vwl));
		add: (self ipa: $@ name: '@' example: 'About' features: #(mdl cnt unr vwl));
		add: (self ipa: $i name: 'i' example: 'bEAd' features: #(hgh fnt unr vwl));
		add: (self ipa: $3 name: 'V"' example: 'bIrd' features: #(lmd cnt unr vwl));
		add: (self ipa: $A name: 'A' example: 'bArd' features: #(low bck unr vwl));
		add: (self ipa: $O name: 'O' example: 'cOrd' features: #(lmd bck rnd vwl));
		add: (self ipa: $u name: 'u' example: 'food' features: #(hgh bck rnd vwl));
		add: (self ipa: $a name: 'a' example: '' features: #(low cnt unr vwl));
		add: (self ipa: $o name: 'o' example: '' features: #(umd bck rnd vwl))! !


!Phoneme class methodsFor: 'examples' stamp: 'len 4/13/98 16:22'!
default
	^ self english! !

!Phoneme class methodsFor: 'examples' stamp: 'len 4/10/98 02:55'!
english
	"Answer the collection of english phonemes."
	^ english! !


Phoneme initialize!


Object subclass: #PhoneticRule
	instanceVariableNames: 'left text right phonemes '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!
!PhoneticRule commentStamp: 'len 5/5/98 01:10' prior: 0!
My instances are letter-to-sound rules for perform automating translation of text to phonetics. (See the TextToPhonemesTranslator class comment too.)

Rules are made up of four parts: (1) left context pattern, (2) the text to match, (3) the right context pattern, and (4) the phonemes to substitute for the matched text.

The translation procedure begins separating each block of letters (apostrophes included) and adding a space on each side. For each unmatched letter in the word, look through the rules where the text to match starts with the letter in the word. If the text to match if found and the right and left contexts patterns also match, output the phonemes for that rule and skip to the next unmatched letter.

Context patterns special characters:
	#	One or more vowels
	:	Zero or more consonants
	^	One consonant
	.	One of B, D, V, G, J, L, M, N, R, W, or Z (voiced consonants)
	%	One of ER, E, ES, ED, ING, ELY (a suffix)	[ONLY FOR RIGHT CONTEXT]
	+	One of E, I or Y (a "front" vowel)
Furthermore, the space character means any separator (space, comma, colon, etc), and any other character means that character it self.

The english example is derived from: "Automatic Translation of English Text to Phonetics by Means of Letter-To-Sound Rules", NRL Report 7948, January 21st, 1976, Naval Research Laboratory, Washington, D.C. Published by the National Technical Information Service as document "AD/A021 929".
!


!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:46'!
left: aString
	left _ aString! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/10/98 05:01'!
phonemes
	^ phonemes! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:47'!
phonemes: aCollection
	phonemes _ aCollection! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:46'!
right: aString
	right _ aString! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/10/98 05:00'!
text
	^ text! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:46'!
text: aString
	text _ aString! !


!PhoneticRule methodsFor: 'testing' stamp: 'len 4/10/98 04:34'!
matches: aString at: anInteger
	^ ((self textMatches: aString at: anInteger)
		and: [self leftMatches: aString at: anInteger])
			and: [self rightMatches: aString at: anInteger]! !


!PhoneticRule methodsFor: 'private' stamp: 'len 4/10/98 23:19'!
leftMatches: aString at: anInteger
	| leftindex textindex pattern |
	left isEmpty ifTrue: [^ true].
	leftindex _ left size.
	textindex _ anInteger - 1.
	[leftindex >= 1 and: [textindex >= 1]] whileTrue: [
		pattern _ left at: leftindex.
		"first check for simple text or apostrophe:"
		(pattern isAlphaNumeric or: [pattern = $'])
			ifTrue: [(aString at: textindex) asLowercase ~= pattern asLowercase
						ifTrue: [^ false].
					textindex _ textindex - 1].
		"space:"
		pattern = Character space
			ifTrue: [(aString at: textindex) isSeparator ifFalse: [^ false].
					textindex _ textindex - 1].
		"one or more vowels:"
		pattern = $#
			ifTrue: [(aString at: textindex) isVowel ifFalse: [^ false].
					textindex _ textindex - 1.
					[textindex >= 1 and: [(aString at: textindex) isVowel]]
						whileTrue: [textindex _ textindex - 1]].
		"zero or more consonants:"
		pattern = $:
			ifTrue: [[textindex >= 1
						and: ['bcdfghjklmnpqrstvwxyz'
								includes: (aString at: textindex) asLowercase]]
							whileTrue: [textindex _ textindex - 1]].
		"one consonant:"
		pattern = $^
			ifTrue: [('bcdfghjklmnpqrstvwxyz' includes: (aString at: textindex))
						ifFalse: [^ false].
					textindex _ textindex - 1].
		"b, d, v, g, j, l, m, n, r, w, z (voiced consonants):"
		pattern = $.
			ifTrue: [('bdvgjlmnrwz' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex - 1].
		"e, i or y (front vowels)"
		pattern = $+
			ifTrue: [('eiy' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex - 1].
		leftindex _ leftindex - 1].
	^ true! !

!PhoneticRule methodsFor: 'private' stamp: 'len 4/13/98 20:49'!
rightMatches: aString at: anInteger
	| rightindex textindex pattern |
	right isEmpty ifTrue: [^ true].
	rightindex _ 1.
	textindex _ anInteger + text size.
	[rightindex <= right size and: [textindex <= aString size]] whileTrue: [
		pattern _ right at: rightindex.
		"first check for simple text or apostrophe:"
		(pattern isAlphaNumeric or: [pattern = $'])
			ifTrue: [(aString at: textindex) asLowercase ~= pattern asLowercase
						ifTrue: [^ false].
					textindex _ textindex + 1].
		"space:"
		pattern = Character space
			ifTrue: [(aString at: textindex) isSeparator ifFalse: [^ false].
					textindex _ textindex + 1].
		"one or more vowels:"
		pattern = $#
			ifTrue: [(aString at: textindex) isVowel ifFalse: [^ false].
					textindex _ textindex + 1.
					[textindex <= aString size and: [(aString at: textindex) isVowel]]
						whileTrue: [textindex _ textindex + 1]].
		"zero or more consonants:"
		pattern = $:
			ifTrue: [[textindex <= aString size
						and: ['bcdfghjklmnpqrstvwxyz'
								includes: (aString at: textindex) asLowercase]]
							whileTrue: [textindex _ textindex + 1]].
		"one consonant:"
		pattern = $^
			ifTrue: [('bcdfghjklmnpqrstvwxyz' includes: (aString at: textindex))
						ifFalse: [^ false].
					textindex _ textindex + 1].
		"b, d, v, g, j, l, m, n, r, w, z (voiced consonants):"
		pattern = $.
			ifTrue: [('bdvgjlmnrwz' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex + 1].
		"e, i or y (front vowels):"
		pattern = $+
			ifTrue: [('eiy' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex + 1].
		"er, e, es, ed, ing, ely (a suffix):"
		pattern = $%
			ifTrue: [(aString at: textindex) asLowercase = $e
						ifTrue: [textindex _ textindex + 1.
								(textindex < aString size and: [(aString at: textindex) asLowercase = $l])
									ifTrue: [textindex _ textindex + 1.
											(textindex < aString size and: [(aString at: textindex) asLowercase = $y])
												ifTrue: [textindex _ textindex + 1]
												ifFalse: [textindex _ textindex - 1]]
									ifFalse: [('rsd' includes: (aString at: textindex) asLowercase)
												ifTrue: [textindex _ textindex + 1]]]
						ifFalse: [(textindex + 2 <= aString size
									and: [(aString at: textindex) asLowercase = $i
										and: [(aString at: textindex + 1) asLowercase = $n
											and: [(aString at: textindex + 2) asLowercase = $g]]])
									ifTrue: [textindex _ textindex + 3]
									ifFalse: [^ false]]].
		rightindex _ rightindex + 1].
	^ true! !

!PhoneticRule methodsFor: 'private' stamp: 'len 4/10/98 23:19'!
textMatches: aString at: anInteger
	text size > (aString size - anInteger + 1) ifTrue: [^ false].
	1 to: text size do: [ :each |
		(text at: each) asLowercase = (aString at: anInteger + each - 1) asLowercase ifFalse: [^ false]].
	^ true! !


!PhoneticRule methodsFor: 'printing' stamp: 'len 4/13/98 20:58'!
printOn: aStream
	aStream
		nextPut: $[; print: left; nextPut: $,; print: text; nextPut: $,; print: right; nextPut: $];
		nextPutAll: ' -> '.
	phonemes isEmpty ifTrue: [aStream nextPutAll: '{}'] ifFalse: [aStream nextPut: $/].
	phonemes do: [ :each | aStream nextPut: each ipa; nextPut: $/]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PhoneticRule class
	instanceVariableNames: ''!

!PhoneticRule class methodsFor: 'instance creation' stamp: 'len 4/30/98 22:15'!
fromArray: anArray phonemes: aCollectionOfPhonemes
	| table codes |
	table _
		#((AX	'@') (AA	'0') (r	r) (EH	e) (EY	eI) (AO	O) (IY	i)
		(l	l) (g	g) (m	m) (n	n) (b	b) (j	j) (k	k)
		(z	z) (s	s) (d	d) (t	t) (v	v) (f	f) (h	h)
		(w	w) (p	p)
		(AE	'&') (AY	aI) (AH	V) (IH	I) (NG	IN) (OW	'@U') (UW	u)
		(OY	oI) (UH	U) (AW	aU)
		(WH	hw) (TH	T) (DH	D) (SH	S) (ZH	Z) (CH	tS) (ER	'3') (y	j)).
	codes _ ''.
	(anArray at: 4) asArrayOfSubstrings do: [ :each |
		codes _ codes, (table detect: [ :one | each = one first]) last].
	codes _ codes asArray.
	^ self new
		left: (anArray at: 1);
		text: (anArray at: 2);
		right: (anArray at: 3);
		phonemes: (codes collect: [ :each | aCollectionOfPhonemes detect: [ :one | one ipa = each]])! !


!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 04:57'!
english
	"Answer the english phonetic rules."
	^ OrderedCollection new
		addAll: self englishPunctuationRules;
		addAll: self englishARules;
		addAll: self englishBRules;
		addAll: self englishCRules;
		addAll: self englishDRules;
		addAll: self englishERules;
		addAll: self englishFRules;
		addAll: self englishGRules;
		addAll: self englishHRules;
		addAll: self englishIRules;
		addAll: self englishJRules;
		addAll: self englishKRules;
		addAll: self englishLRules;
		addAll: self englishMRules;
		addAll: self englishNRules;
		addAll: self englishORules;
		addAll: self englishPRules;
		addAll: self englishQRules;
		addAll: self englishRRules;
		addAll: self englishSRules;
		addAll: self englishTRules;
		addAll: self englishURules;
		addAll: self englishVRules;
		addAll: self englishWRules;
		addAll: self englishXRules;
		addAll: self englishYRules;
		addAll: self englishZRules;
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:10'!
englishARules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'a'		' '		'AX') phonemes: phonemes);
		add: (self fromArray: #(' '		'are'	' '		'AA r') phonemes: phonemes);
		add: (self fromArray: #(' '		'ar'		'o'		'AX r') phonemes: phonemes);
		add: (self fromArray: #(''		'ar'		'#'		'EH r') phonemes: phonemes);
		add: (self fromArray: #('^'		'as'		'#'		'EY s') phonemes: phonemes);
		add: (self fromArray: #(''		'a'		'wa'	'AX') phonemes: phonemes);
		add: (self fromArray: #(''		'aw'	''		'AO') phonemes: phonemes);
		add: (self fromArray: #(' :'		'any'	''		'EH n IY') phonemes: phonemes);
		add: (self fromArray: #(''		'a'		'^+#'	'EY') phonemes: phonemes);
		add: (self fromArray: #('#:'		'ally'	''		'AX l IY') phonemes: phonemes);
		add: (self fromArray: #(' '		'al'		'#'		'AX l') phonemes: phonemes);
		add: (self fromArray: #(''		'again'	''		'AX g EH n') phonemes: phonemes);
		add: (self fromArray: #('#:'		'ag'		'e'		'IH j') phonemes: phonemes);
		add: (self fromArray: #(''		'a'		'^+:#'	'AE') phonemes: phonemes);
		add: (self fromArray: #(' :'		'a'		'^+ '	'EY') phonemes: phonemes);
		add: (self fromArray: #(' '		'arr'	''		'AX r') phonemes: phonemes);
		add: (self fromArray: #(''		'arr'	''		'AE r') phonemes: phonemes);
		add: (self fromArray: #(' :'		'ar'		' '		'AA r') phonemes: phonemes);
		add: (self fromArray: #(''		'ar'		' '		'ER') phonemes: phonemes);
		add: (self fromArray: #(''		'ar'		''		'AA r') phonemes: phonemes);
		add: (self fromArray: #(''		'air'	''		'EH r') phonemes: phonemes);
		add: (self fromArray: #(''		'ai'		''		'EY') phonemes: phonemes);
		add: (self fromArray: #(''		'ay'		''		'EY') phonemes: phonemes);
		add: (self fromArray: #(''		'au'		''		'AO') phonemes: phonemes);
		add: (self fromArray: #('#:'		'al'		' '		'AX l') phonemes: phonemes);
		add: (self fromArray: #('#:'		'als'	' '		'AX l z') phonemes: phonemes);
		add: (self fromArray: #(''		'alk'	''		'AO k') phonemes: phonemes);
		add: (self fromArray: #(''		'al'		'^'		'AO l') phonemes: phonemes);
		add: (self fromArray: #(' :'		'able'	''		'EY b AX l') phonemes: phonemes);
		add: (self fromArray: #(''		'able'	''		'AX b AX l') phonemes: phonemes);
		add: (self fromArray: #(''		'ang'	'+'		'EY n j') phonemes: phonemes);
		add: (self fromArray: #(''		'a'		''		'AE') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:15'!
englishBRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(' '		'be'		'^#'		'b IH') phonemes: phonemes);
		add: (self fromArray: #(''		'being'	''		'b IY IH NG') phonemes: phonemes);
		add: (self fromArray: #(' '		'both'	''		'b OW TH') phonemes: phonemes);
		add: (self fromArray: #(' '		'bus'	'#'		'b IH z') phonemes: phonemes);
		add: (self fromArray: #(''		'buil'	''		'b IH l') phonemes: phonemes);
		add: (self fromArray: #(''		'b'		''		'b') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 16:34'!
englishCRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(' '		'ch'		'^'		'k') phonemes: phonemes);
		add: (self fromArray: #('^e'		'ch'		''		'k') phonemes: phonemes);
		add: (self fromArray: #(''		'ch'		''		'CH') phonemes: phonemes);
		add: (self fromArray: #(' s'		'ci'		'#'		's AY') phonemes: phonemes);
		add: (self fromArray: #(''		'ci'		'a'		'SH') phonemes: phonemes);
		add: (self fromArray: #(''		'ci'		'o'		'SH') phonemes: phonemes);
		add: (self fromArray: #(''		'ci'		'en'		'SH') phonemes: phonemes);
		add: (self fromArray: #(''		'c'		'+'		's') phonemes: phonemes);
		add: (self fromArray: #(''		'ck'		''		'k') phonemes: phonemes);
		add: (self fromArray: #(''		'com'	'%'		'k AH m') phonemes: phonemes);
		add: (self fromArray: #(''		'c'		''		'k') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:23'!
englishDRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #('#:'		'ded'	' '		'd IH d') phonemes: phonemes);
		add: (self fromArray: #('.e'		'd'		' '		'd') phonemes: phonemes);
		add: (self fromArray: #('#:^e'	'd'		' '		't') phonemes: phonemes);
		add: (self fromArray: #(' '		'de'		''		'') phonemes: phonemes);
		add: (self fromArray: #(' '		'do'		' '		'd UW') phonemes: phonemes);
		add: (self fromArray: #(' '		'does'	''		'd AH z') phonemes: phonemes);
		add: (self fromArray: #(' '		'doing'	''		'd UW IH NG') phonemes: phonemes);
		add: (self fromArray: #(' '		'dow'	''		'd AW') phonemes: phonemes);
		add: (self fromArray: #(''		'du'		'a'		'j UW') phonemes: phonemes);
		add: (self fromArray: #(''		'd'		''		'd') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:39'!
englishERules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #('#:'		'e'		' '		'') phonemes: phonemes);
		add: (self fromArray: #(':^'		'e'		' '		'') phonemes: phonemes);
		add: (self fromArray: #(' :'		'e'		' '		'IY') phonemes: phonemes);
		add: (self fromArray: #('#'		'ed'		' '		'd') phonemes: phonemes);
		add: (self fromArray: #('#:'		'e'		'd'		'') phonemes: phonemes);
		add: (self fromArray: #(''		'ev'		'er'		'EH v') phonemes: phonemes);
		add: (self fromArray: #(''		'e'		'^%'		'IY') phonemes: phonemes);
		add: (self fromArray: #(''		'eri'	'#'		'IY r IY') phonemes: phonemes);
		add: (self fromArray: #(''		'eri'	''		'EH r IH') phonemes: phonemes);
		add: (self fromArray: #('#:'		'er'		'#'		'ER') phonemes: phonemes);
		add: (self fromArray: #(''		'er'		'#'		'EH r') phonemes: phonemes);
		add: (self fromArray: #(''		'er'		''		'ER') phonemes: phonemes);
		add: (self fromArray: #(' '		'even'	''		'IY v EH n') phonemes: phonemes);
		add: (self fromArray: #('#:'		'e'		'w'		'') phonemes: phonemes);
		add: (self fromArray: #('t'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('s'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('r'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('d'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('l'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('z'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('n'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('j'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('th'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('ch'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #('sh'		'ew'	''		'UW') phonemes: phonemes);
		add: (self fromArray: #(''		'ew'	''		'y UW') phonemes: phonemes);
		add: (self fromArray: #(''		'e'		'o'		'IY') phonemes: phonemes);
		add: (self fromArray: #('#:s'	'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:c'	'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:g'	'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:z'	'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:x'	'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:j'		'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:ch'	'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:sh'	'es'		' '		'IH z') phonemes: phonemes);
		add: (self fromArray: #('#:'		'e'		's'		'') phonemes: phonemes);
		add: (self fromArray: #('#:'		'ely'	' '		'l IY') phonemes: phonemes);
		add: (self fromArray: #('#:'		'ement'	''		'm EH n t') phonemes: phonemes);
		add: (self fromArray: #(''		'eful'	''		'f UH l') phonemes: phonemes);
		add: (self fromArray: #(''		'ee'		''		'IY') phonemes: phonemes);
		add: (self fromArray: #(''		'earn'	''		'ER n') phonemes: phonemes);
		add: (self fromArray: #(' '		'ear'	'^'		'ER') phonemes: phonemes);
		add: (self fromArray: #(''		'ead'	''		'EH d') phonemes: phonemes);
		add: (self fromArray: #('#:'		'ea'		' '		'IY AX') phonemes: phonemes);
		add: (self fromArray: #(''		'ea'		'su'		'EH') phonemes: phonemes);
		add: (self fromArray: #(''		'ea'		''		'IY') phonemes: phonemes);
		add: (self fromArray: #(''		'eigh'	''		'EY') phonemes: phonemes);
		add: (self fromArray: #(''		'ei'		''		'IY') phonemes: phonemes);
		add: (self fromArray: #(' '		'eye'	''		'AY') phonemes: phonemes);
		add: (self fromArray: #(''		'ey'		''		'IY') phonemes: phonemes);
		add: (self fromArray: #(''		'eu'		''		'y UW') phonemes: phonemes);
		add: (self fromArray: #(''		'e'		''		'EH') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:40'!
englishFRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'ful'	''		'f UH l') phonemes: phonemes);
		add: (self fromArray: #(''		'f'		''		'f') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:43'!
englishGRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'giv'	''		'g IY v') phonemes: phonemes);
		add: (self fromArray: #(' '		'g'		'i^'		'g') phonemes: phonemes);
		add: (self fromArray: #(''		'ge'		't'		'g EH') phonemes: phonemes);
		add: (self fromArray: #('su'		'gges'	''		'g j EH s') phonemes: phonemes);
		add: (self fromArray: #(''		'gg'		''		'g') phonemes: phonemes);
		add: (self fromArray: #(' b#'	'g'		''		'g') phonemes: phonemes);
		add: (self fromArray: #(''		'g'		'+'		'j') phonemes: phonemes);
		add: (self fromArray: #(''		'great'	''		'g r EY t') phonemes: phonemes);
		add: (self fromArray: #('#'		'gh'		''		'') phonemes: phonemes);
		add: (self fromArray: #(''		'g'		''		'g') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:45'!
englishHRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(' '		'hav'	''		'h AE v') phonemes: phonemes);
		add: (self fromArray: #(' '		'here'	''		'h IY r') phonemes: phonemes);
		add: (self fromArray: #(' '		'hour'	''		'AW ER') phonemes: phonemes);
		add: (self fromArray: #(''		'how'	''		'h AW') phonemes: phonemes);
		add: (self fromArray: #(''		'h'		'#'		'h') phonemes: phonemes);
		add: (self fromArray: #(''		'h'		''		'') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:54'!
englishIRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(' '		'in'		''		'IH n') phonemes: phonemes);
		add: (self fromArray: #(' '		'i'		' '		'AY') phonemes: phonemes);
		add: (self fromArray: #(''		'in'		'd'		'AY n') phonemes: phonemes);
		add: (self fromArray: #(''		'ier'	''		'IY ER') phonemes: phonemes);
		add: (self fromArray: #('#:r'	'ied'	''		'IY d') phonemes: phonemes);
		add: (self fromArray: #(''		'ied'	' '		'AY d') phonemes: phonemes);
		add: (self fromArray: #(''		'ien'	''		'IY EH n') phonemes: phonemes);
		add: (self fromArray: #(''		'ie'		't'		'AY EH') phonemes: phonemes);
		add: (self fromArray: #(' :'		'i'		'%'		'AY') phonemes: phonemes);
		add: (self fromArray: #(''		'i'		'%'		'IY') phonemes: phonemes);
		add: (self fromArray: #(''		'ie'		''		'IY') phonemes: phonemes);
		add: (self fromArray: #(''		'i'		'^+:#'	'IH') phonemes: phonemes);
		add: (self fromArray: #(''		'ir'		'#'		'AY r') phonemes: phonemes);
		add: (self fromArray: #(''		'iz'		'%'		'AY z') phonemes: phonemes);
		add: (self fromArray: #(''		'is'		'%'		'AY z') phonemes: phonemes);
		add: (self fromArray: #(''		'i'		'd%'		'AY') phonemes: phonemes);
		add: (self fromArray: #('+^'		'i'		'^+'		'IH') phonemes: phonemes);
		add: (self fromArray: #(''		'i'		't%'		'AY') phonemes: phonemes);
		add: (self fromArray: #('#:^'	'i'		'^+'		'IH') phonemes: phonemes);
		add: (self fromArray: #(''		'i'		'^+'		'AY') phonemes: phonemes);
		add: (self fromArray: #(''		'ir'		''		'ER') phonemes: phonemes);
		add: (self fromArray: #(''		'igh'	''		'AY') phonemes: phonemes);
		add: (self fromArray: #(''		'ild'		''		'AY l d') phonemes: phonemes);
		add: (self fromArray: #(''		'ign'	' '		'AY n') phonemes: phonemes);
		add: (self fromArray: #(''		'ign'	'^'		'AY n') phonemes: phonemes);
		add: (self fromArray: #(''		'ign'	'%'		'AY n') phonemes: phonemes);
		add: (self fromArray: #(''		'ique'	''		'IY k') phonemes: phonemes);
		add: (self fromArray: #(''		'i'		''		'IH') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:55'!
englishJRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'j'		''		'j') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:56'!
englishKRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'k'		'n'		'') phonemes: phonemes);
		add: (self fromArray: #(''		'k'		''		'k') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:57'!
englishLRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'lo'		'c#'		'l OW') phonemes: phonemes);
		add: (self fromArray: #('l'		'l'		''		'') phonemes: phonemes);
		add: (self fromArray: #('#:^'	'l'		'%'		'AX l') phonemes: phonemes);
		add: (self fromArray: #(''		'lead'	''		'l IY d') phonemes: phonemes);
		add: (self fromArray: #(''		'l'		''		'l') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:58'!
englishMRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'mov'	''		'm UW v') phonemes: phonemes);
		add: (self fromArray: #(''		'm'		''		'm') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:00'!
englishNRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #('e'		'ng'		'+'		'n j') phonemes: phonemes);
		add: (self fromArray: #(''		'ng'		'r'		'NG g') phonemes: phonemes);
		add: (self fromArray: #(''		'ng'		'#'		'NG g') phonemes: phonemes);
		add: (self fromArray: #(''		'ngl'	'%'		'NG g AX l') phonemes: phonemes);
		add: (self fromArray: #(''		'nk'		''		'NG k') phonemes: phonemes);
		add: (self fromArray: #(' '		'now'	' '		'n AW') phonemes: phonemes);
		add: (self fromArray: #(''		'n'		''		'n') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:16'!
englishORules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'of'		' '		'AX v') phonemes: phonemes);
		add: (self fromArray: #(''		'orough'	''	'ER OW') phonemes: phonemes);
		add: (self fromArray: #('#:'		'or'		' '		'ER') phonemes: phonemes);
		add: (self fromArray: #('#:'		'ors'	' '		'ER z') phonemes: phonemes);
		add: (self fromArray: #(''		'or'		''		'AO r') phonemes: phonemes);
		add: (self fromArray: #(' '		'one'	''		'w AH n') phonemes: phonemes);
		add: (self fromArray: #(''		'ow'	''		'OW') phonemes: phonemes);
		add: (self fromArray: #(' '		'over'	''		'OW v ER') phonemes: phonemes);
		add: (self fromArray: #(''		'ov'		''		'AH v') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		'^%'		'OW') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		'^en'	'OW') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		'^i#'	'OW') phonemes: phonemes);
		add: (self fromArray: #(''		'ol'		'd'		'OW l') phonemes: phonemes);
		add: (self fromArray: #(''		'ought'	''		'AO t') phonemes: phonemes);
		add: (self fromArray: #(''		'ough'	''		'AH f') phonemes: phonemes);
		add: (self fromArray: #(' '		'ou'		''		'AW') phonemes: phonemes);
		add: (self fromArray: #('h'		'ou'		's#'		'AW') phonemes: phonemes);
		add: (self fromArray: #(''		'ous'	''		'AX s') phonemes: phonemes);
		add: (self fromArray: #(''		'our'	''		'AO r') phonemes: phonemes);
		add: (self fromArray: #(''		'ould'	''		'UH d') phonemes: phonemes);
		add: (self fromArray: #('^'		'ou'		'^l'		'AH') phonemes: phonemes);
		add: (self fromArray: #(''		'oup'	''		'UW p') phonemes: phonemes);
		add: (self fromArray: #(''		'ou'		''		'AW') phonemes: phonemes);
		add: (self fromArray: #(''		'oy'		''		'OY') phonemes: phonemes);
		add: (self fromArray: #(''		'oing'	''		'OW IH NG') phonemes: phonemes);
		add: (self fromArray: #(''		'oi'		''		'OY') phonemes: phonemes);
		add: (self fromArray: #(''		'oor'	''		'AO r') phonemes: phonemes);
		add: (self fromArray: #(''		'ook'	''		'UH k') phonemes: phonemes);
		add: (self fromArray: #(''		'ood'	''		'UH d') phonemes: phonemes);
		add: (self fromArray: #(''		'oo'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		'e'		'OW') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		' '		'OW') phonemes: phonemes);
		add: (self fromArray: #(''		'oa'		''		'OW') phonemes: phonemes);
		add: (self fromArray: #(' '		'only'	''		'OW n l IY') phonemes: phonemes);
		add: (self fromArray: #(' '		'once'	''		'w AH n s') phonemes: phonemes);
		add: (self fromArray: #(''		'on''t'	''		'OW n t') phonemes: phonemes);
		add: (self fromArray: #('c'		'o'		'n'		'AA') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		'ng'		'AO') phonemes: phonemes);
		add: (self fromArray: #(' :^'		'o'		'n'		'AH') phonemes: phonemes);
		add: (self fromArray: #('i'		'on'		''		'AX n') phonemes: phonemes);
		add: (self fromArray: #('#:'		'on'		' '		'AX n') phonemes: phonemes);
		add: (self fromArray: #('#^'		'on'		''		'AX n') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		'st '		'OW') phonemes: phonemes);
		add: (self fromArray: #(''		'of'		'^'		'AO f') phonemes: phonemes);
		add: (self fromArray: #(''		'other'	''		'AH DH ER') phonemes: phonemes);
		add: (self fromArray: #(''		'oss'	' '		'AO s') phonemes: phonemes);
		add: (self fromArray: #('#:^'	'om'	''		'AH m') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		''		'AA') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:17'!
englishPRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'ph'		''		'f') phonemes: phonemes);
		add: (self fromArray: #(''		'peop'	''		'p IY p') phonemes: phonemes);
		add: (self fromArray: #(''		'pow'	''		'p AW') phonemes: phonemes);
		add: (self fromArray: #(''		'put'	' '		'p UH t') phonemes: phonemes);
		add: (self fromArray: #(''		'p'		''		'p') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 01:06'!
englishPunctuationRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #('.'		'''s'		''		'z') phonemes: phonemes);
		add: (self fromArray: #('#:.e'	'''s'		''		'z') phonemes: phonemes);
		add: (self fromArray: #('#'		'''s'		''		'z') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:18'!
englishQRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'quar'	''		'k w AO r') phonemes: phonemes);
		add: (self fromArray: #(''		'qu'		''		'k w') phonemes: phonemes);
		add: (self fromArray: #(''		'q'		''		'k') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:19'!
englishRRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		're'		'^#'		'r IY') phonemes: phonemes);
		add: (self fromArray: #(''		'r'		''		'r') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 04:55'!
englishSRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'sh'		''		'SH') phonemes: phonemes);
		add: (self fromArray: #('#'		'sion'	''		'ZH AX n') phonemes: phonemes);
		add: (self fromArray: #(''		'some'	''		's AH m') phonemes: phonemes);
		add: (self fromArray: #('#'		'sur'	'#'		'ZH ER') phonemes: phonemes);
		add: (self fromArray: #(''		'sur'	'#'		'SH ER') phonemes: phonemes);
		add: (self fromArray: #('#'		'su'		'#'		'ZH UW') phonemes: phonemes);
		add: (self fromArray: #('#'		'ssu'	'#'		'SH UW') phonemes: phonemes);
		add: (self fromArray: #('#'		'sed'	' '		'z d') phonemes: phonemes);
		add: (self fromArray: #('#'		's'		'#'		'z') phonemes: phonemes);
		add: (self fromArray: #(''		'said'	''		's EH d') phonemes: phonemes);
		add: (self fromArray: #('^'		'sion'	''		'SH AX n') phonemes: phonemes);
		add: (self fromArray: #(''		's'		's'		'') phonemes: phonemes);
		add: (self fromArray: #('.'		's'		' '		'z') phonemes: phonemes);
		add: (self fromArray: #('#:.e'	's'		' '		'z') phonemes: phonemes);
		add: (self fromArray: #('#:^##'	's'		' '		'z') phonemes: phonemes);
		add: (self fromArray: #('#:^#'	's'		' '		's') phonemes: phonemes);
		add: (self fromArray: #('u'		's'		' '		's') phonemes: phonemes);
		add: (self fromArray: #(' :#'	's'		' '		'z') phonemes: phonemes);
		add: (self fromArray: #(' '		'sch'	''		's k') phonemes: phonemes);
		add: (self fromArray: #(''		's'		'c+'		'') phonemes: phonemes);
		add: (self fromArray: #('#'		'sm'		''		'z m') phonemes: phonemes);
		add: (self fromArray: #('#'		'sn'		''''		'z AX n') phonemes: phonemes);
		add: (self fromArray: #(''		's'		''		's') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:37'!
englishTRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(' '		'the'	' '		'DH AX') phonemes: phonemes);
		add: (self fromArray: #(''		'to'		' '		't UW') phonemes: phonemes);
		add: (self fromArray: #(''		'that'	' '		'DH AE t') phonemes: phonemes);
		add: (self fromArray: #(' '		'this'	' '		'DH IH s') phonemes: phonemes);
		add: (self fromArray: #(' '		'they'	''		'DH EY') phonemes: phonemes);
		add: (self fromArray: #(' '		'there'	''		'DH EH r') phonemes: phonemes);
		add: (self fromArray: #(''		'ther'	''		'DH ER') phonemes: phonemes);
		add: (self fromArray: #(''		'their'	''		'DH EH r') phonemes: phonemes);
		add: (self fromArray: #(' '		'than'	' '		'DH AE n') phonemes: phonemes);
		add: (self fromArray: #(' '		'them'	' '		'DH EH m') phonemes: phonemes);
		add: (self fromArray: #(''		'these'	' '		'DH IY z') phonemes: phonemes);
		add: (self fromArray: #(' '		'then'	''		'DH EH n') phonemes: phonemes);
		add: (self fromArray: #(''		'through'	''	'TH r UW') phonemes: phonemes);
		add: (self fromArray: #(''		'those'	''		'DH OW z') phonemes: phonemes);
		add: (self fromArray: #(''		'though'	' '	'DH OW') phonemes: phonemes);
		add: (self fromArray: #(' '		'thus'	''		'DH AH s') phonemes: phonemes);
		add: (self fromArray: #(''		'th'		''		'TH') phonemes: phonemes);
		add: (self fromArray: #('#:'		'ted'	' '		't IH d') phonemes: phonemes);
		add: (self fromArray: #('s'		'ti'		'#n'	'CH') phonemes: phonemes);
		add: (self fromArray: #(''		'ti'		'o'		'SH') phonemes: phonemes);
		add: (self fromArray: #(''		'ti'		'a'		'SH') phonemes: phonemes);
		add: (self fromArray: #(''		'tien'	''		'SH AX n') phonemes: phonemes);
		add: (self fromArray: #(''		'tur'	'#'		'CH ER') phonemes: phonemes);
		add: (self fromArray: #(''		'tu'		'a'		'CH UW') phonemes: phonemes);
		add: (self fromArray: #(''		'two'	''		't UW') phonemes: phonemes);
		add: (self fromArray: #(''		't'		''		't') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:44'!
englishURules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(' '		'un'	'i'		'y UW n') phonemes: phonemes);
		add: (self fromArray: #(' '		'un'	''		'AH n') phonemes: phonemes);
		add: (self fromArray: #(' '		'upon'	''		'AX p AO n') phonemes: phonemes);
		add: (self fromArray: #('t'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('s'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('r'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('d'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('l'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('z'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('n'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('j'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('th'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('ch'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #('sh'		'ur'		'#'		'UH r') phonemes: phonemes);
		add: (self fromArray: #(''		'ur'		'#'		'y UH r') phonemes: phonemes);
		add: (self fromArray: #(''		'ur'		''		'ER') phonemes: phonemes);
		add: (self fromArray: #(''		'u'		'^'		'AH') phonemes: phonemes);
		add: (self fromArray: #(''		'u'		'^^'		'AH') phonemes: phonemes);
		add: (self fromArray: #(''		'uy'	''		'AY') phonemes: phonemes);
		add: (self fromArray: #(' g'		'u'		'#'		'') phonemes: phonemes);
		add: (self fromArray: #('g'		'u'		'%'		'') phonemes: phonemes);
		add: (self fromArray: #('#n'	'u'		''		'y UW') phonemes: phonemes);
		add: (self fromArray: #('t'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('s'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('r'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('d'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('l'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('z'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('n'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('j'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('th'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('ch'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #('sh'		'u'		''		'UW') phonemes: phonemes);
		add: (self fromArray: #(''		'u'		''		'y UW') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:48'!
englishVRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'view'	''		'v y UW') phonemes: phonemes);
		add: (self fromArray: #(''		'v'		''		'v') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:48'!
englishWRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(' '		'were'	''		'w ER') phonemes: phonemes);
		add: (self fromArray: #(''		'wa'	's'		'w AA') phonemes: phonemes);
		add: (self fromArray: #(''		'wa'	't'		'w AA') phonemes: phonemes);
		add: (self fromArray: #(''		'where'	''		'WH EH r') phonemes: phonemes);
		add: (self fromArray: #(''		'what'	''		'WH AA t') phonemes: phonemes);
		add: (self fromArray: #(''		'whol'	''		'h OW l') phonemes: phonemes);
		add: (self fromArray: #(''		'who'	''		'h UW') phonemes: phonemes);
		add: (self fromArray: #(''		'wh'	''		'WH') phonemes: phonemes);
		add: (self fromArray: #(''		'war'	''		'w AO r') phonemes: phonemes);
		add: (self fromArray: #(''		'wor'	'^'		'w ER') phonemes: phonemes);
		add: (self fromArray: #(''		'wr'	''		'r') phonemes: phonemes);
		add: (self fromArray: #(''		'w'		''		'w') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 04:55'!
englishXRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'x'		''		'k s') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:52'!
englishYRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'young'	''		'y AH NG') phonemes: phonemes);
		add: (self fromArray: #(' '		'you'	''		'y UW') phonemes: phonemes);
		add: (self fromArray: #(' '		'yes'	''		'y EH s') phonemes: phonemes);
		add: (self fromArray: #(' '		'y'		''		'y') phonemes: phonemes);
		add: (self fromArray: #('#:^'	'y'		' '		'IY') phonemes: phonemes);
		add: (self fromArray: #('#:^'	'y'		'i'		'IY') phonemes: phonemes);
		add: (self fromArray: #(' :'		'y'		''		'AY') phonemes: phonemes);
		add: (self fromArray: #(' :'		'y'		'#'		'AY') phonemes: phonemes);
		add: (self fromArray: #(' :'		'y'		'^+:#'	'IH') phonemes: phonemes);
		add: (self fromArray: #(' :'		'y'		'^#'		'AY') phonemes: phonemes);
		add: (self fromArray: #(''		'y'		''		'IH') phonemes: phonemes);
		yourself! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 4/10/98 02:53'!
englishZRules
	| phonemes |
	phonemes _ Phoneme english.
	^ OrderedCollection new
		add: (self fromArray: #(''		'z'		''		'z') phonemes: phonemes);
		yourself! !


!PhoneticRule class methodsFor: 'examples' stamp: 'len 4/15/98 23:44'!
spanish
	"Answer the spanish phonetic rules."
	| phonemes |
	phonemes _ Phoneme spanish.
	^ OrderedCollection new
		add: (self fromArray: #(''		'a'		''		'a') phonemes: phonemes);
		add: (self fromArray: #(''		'b'		''		'b') phonemes: phonemes);
		add: (self fromArray: #(''		'ch'		''		'ch') phonemes: phonemes);
		add: (self fromArray: #(''		'c'		'i'		'z') phonemes: phonemes);
		add: (self fromArray: #(''		'c'		'e'		'z') phonemes: phonemes);
		add: (self fromArray: #(''		'c'		''		'k') phonemes: phonemes);
		add: (self fromArray: #(''		'd'		''		'd') phonemes: phonemes);
		add: (self fromArray: #(''		'e'		''		'e') phonemes: phonemes);
		add: (self fromArray: #(''		'f'		''		'f') phonemes: phonemes);
		add: (self fromArray: #(''		'g'		'e'		'j') phonemes: phonemes);
		add: (self fromArray: #(''		'g'		'i'		'j') phonemes: phonemes);
		add: (self fromArray: #(''		'gu'		'e'		'g') phonemes: phonemes);
		add: (self fromArray: #(''		'gu'		'i'		'g') phonemes: phonemes);
		add: (self fromArray: #(''		'g'		''		'g') phonemes: phonemes);
		add: (self fromArray: #(''		'h'		''		'') phonemes: phonemes);
		add: (self fromArray: #(''		'i'		''		'i') phonemes: phonemes);
		add: (self fromArray: #(''		'j'		''		'j') phonemes: phonemes);
		add: (self fromArray: #(''		'k'		''		'k') phonemes: phonemes);
		add: (self fromArray: #(''		'll'		''		'y') phonemes: phonemes);
		add: (self fromArray: #(''		'l'		''		'l') phonemes: phonemes);
		add: (self fromArray: #(''		'm'		''		'm') phonemes: phonemes);
		add: (self fromArray: #(''		'n'		''		'n') phonemes: phonemes);
		add: (self fromArray: #(''		'o'		''		'o') phonemes: phonemes);
		add: (self fromArray: #(''		'p'		''		'p') phonemes: phonemes);
		add: (self fromArray: #(''		'qu'		''		'k') phonemes: phonemes);
		add: (self fromArray: #(''		'rr'		''		'rr') phonemes: phonemes);
		add: (self fromArray: #(''		'r'		''		'r') phonemes: phonemes);
		add: (self fromArray: #(''		's'		''		's') phonemes: phonemes);
		add: (self fromArray: #(''		't'		''		't') phonemes: phonemes);
		add: (self fromArray: #(''		'u'		''		'u') phonemes: phonemes);
		add: (self fromArray: #(''		'v'		''		'v') phonemes: phonemes);
		add: (self fromArray: #(''		'w'		''		'w') phonemes: phonemes);
		add: (self fromArray: #(''		'x'		''		'k s') phonemes: phonemes);
		add: (self fromArray: #(''		'y'		' '		'i') phonemes: phonemes);
		add: (self fromArray: #(''		'y'		''		'y') phonemes: phonemes);
		add: (self fromArray: #(''		'z'		''		'z') phonemes: phonemes);
		yourself! !


Object subclass: #SimpleVoiceFilter
	instanceVariableNames: 'a b v '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!

!SimpleVoiceFilter methodsFor: 'accessing' stamp: 'len 4/12/98 19:56'!
a: aNumber b: bNumber v: vNumber
	a _ aNumber.
	b _ bNumber.
	v _ vNumber! !

!SimpleVoiceFilter methodsFor: 'accessing' stamp: 'len 4/12/98 19:55'!
value: aNumber
	^ v _ a * aNumber + (b * v)! !


Object subclass: #SpeechProcessor
	instanceVariableNames: 'translator interpolator synthesizer lastSound '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!
!SpeechProcessor commentStamp: 'len 5/5/98 01:10' prior: 0!
My instances are capable of pronounce text strings unsing the SoundPlayer. Each SpeechProcessor have a translator (TextToVoiceElementsTranslator), an interpolator (VoiceInterpolator) and a synthesizer (VoiceSynthesizer).

Provided a text string, the translator extracts a collection of VoiceElements from the string, then the interpolator take as input the VoiceElements and join them softly producing a collection of VoiceFrames as it's output, and finally the synthesizer use the VoiceFrames to produce a sound, which is played.

For an example try this
	SpeechProcessor default say: 'I love Squeak.'.
!


!SpeechProcessor methodsFor: 'accessing-private' stamp: 'len 4/13/98 19:36'!
interpolator: aVoiceInterpolator
	interpolator _ aVoiceInterpolator! !

!SpeechProcessor methodsFor: 'accessing-private' stamp: 'len 4/13/98 19:36'!
synthesizer: aVoiceSynthesizer
	synthesizer _ aVoiceSynthesizer! !

!SpeechProcessor methodsFor: 'accessing-private' stamp: 'len 4/13/98 19:35'!
translator: aTextToVoiceElementsTranslator
	translator _ aTextToVoiceElementsTranslator! !


!SpeechProcessor methodsFor: 'accessing' stamp: 'len 4/15/98 05:48'!
lastSound
	^ lastSound! !

!SpeechProcessor methodsFor: 'accessing' stamp: 'len 4/15/98 05:48'!
replay
	lastSound play! !

!SpeechProcessor methodsFor: 'accessing' stamp: 'len 4/15/98 05:48'!
say: aString
	| elements frames |
	elements _ translator text: aString; elements.
	frames _ interpolator framesFromVoiceElements: elements.
	lastSound _ synthesizer soundForFrames: frames.
	lastSound play! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SpeechProcessor class
	instanceVariableNames: ''!

!SpeechProcessor class methodsFor: 'examples' stamp: 'len 4/13/98 19:40'!
default
	^ self new
		translator: TextToVoiceElementsTranslator default;
		interpolator: VoiceInterpolator new;
		synthesizer: VoiceSynthesizer default! !


Object subclass: #TextToPhonemesTranslator
	instanceVariableNames: 'rules text index phonemes unmatchedPositions '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!
!TextToPhonemesTranslator commentStamp: 'len 5/5/98 01:10' prior: 0!
My instances perform automating translation of text to phonetics. (See the PhoneticRule class comment too.)

Each translator must have a collection of PhoneticRules.

Rules are made up of four parts: (1) left context pattern, (2) the text to match, (3) the right context pattern, and (4) the phonemes to substitute for the matched text.

The translation procedure begins when a text is provided to a translator. For each unmatched letter in the word, look through the rules where the text to match starts with the letter in the word. If one matching rule is found, then the rule is applied, writing the corresponding phonemes in the output and moving forward to the next unmatched position. If no rule is found for a position in the text, then the unmatched position is logged. At the end of the translation, the phonemes extracted are provided in a collection and so are the unmatched positions.
!


!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/13/98 16:26'!
addAllPhonemes: aCollection
	phonemes addAll: aCollection! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/10/98 03:50'!
done
	^ index > text size! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:28'!
phonemes
	self done ifFalse: [self translate].
	^ phonemes! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:14'!
reset
	index _ 1.
	phonemes _ OrderedCollection new.
	unmatchedPositions _ OrderedCollection new! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/13/98 20:12'!
ruleNotFound
	self halt.
	unmatchedPositions add: index.
	index _ index + 1! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/10/98 05:10'!
rules: aCollection
	rules _ aCollection! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/10/98 05:04'!
skipBlanks
	[index <= text size and: [(text at: index) isSeparator]]
		whileTrue: [index _ index + 1]! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:14'!
text: aString
	text _ aString.
	self reset! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/10/98 03:52'!
translate
	[self done]
		whileFalse: [self skipBlanks.
					self done ifFalse: [self translateWord]]! !

!TextToPhonemesTranslator methodsFor: 'all' stamp: 'len 4/13/98 21:11'!
translateWord
	| rule |
	rule _ rules detect: [ :one | one matches: text at: index] ifNone: [^ self ruleNotFound].
	"Transcript cr; show: rule printString."
	index _ index + rule text size.
	self addAllPhonemes: rule phonemes! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TextToPhonemesTranslator class
	instanceVariableNames: ''!

!TextToPhonemesTranslator class methodsFor: 'examples' stamp: 'len 4/13/98 16:21'!
default
	^ self english! !

!TextToPhonemesTranslator class methodsFor: 'examples' stamp: 'len 4/10/98 04:38'!
english
	"Answer the english phonemes extractor."
	^ self new rules: PhoneticRule english! !


TextToPhonemesTranslator subclass: #TextToVoiceElementsTranslator
	instanceVariableNames: 'elements phonemesToElementsDictionary spaceElement endElement '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!
!TextToVoiceElementsTranslator commentStamp: 'len 5/5/98 01:10' prior: 0!
My instances perform automating translation of text to voice elements. I am implemented as a specialization of the TextToPhonemesTranslator. (See the VoiceElement class comment too.)

Each translator must have a dictionary from Phonemes to collections of VoiceElements.!


!TextToVoiceElementsTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:20'!
addAllPhonemes: aCollection
	super addAllPhonemes: aCollection.
	aCollection do: [ :each | elements addAll: (phonemesToElementsDictionary at: each)]! !

!TextToVoiceElementsTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:27'!
elements
	self done ifFalse: [self translate].
	^ elements! !

!TextToVoiceElementsTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:18'!
endElement: aVoiceElement
	endElement _ aVoiceElement! !

!TextToVoiceElementsTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:18'!
phonemesToElementsDictionary: aDictionary
	phonemesToElementsDictionary _ aDictionary! !

!TextToVoiceElementsTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:15'!
reset
	super reset.
	elements _ OrderedCollection new! !

!TextToVoiceElementsTranslator methodsFor: 'all' stamp: 'len 5/1/98 20:33'!
skipBlanks
	(self done not and: [(text at: index) = Character space])
		ifTrue: [elements add: spaceElement; add: spaceElement. index _ index + 1].
	(self done not and: [(text at: index) = $,])
		ifTrue: [elements add: spaceElement; add: spaceElement. index _ index + 1].
	(self done not and: [(text at: index) = $;])
		ifTrue: [elements add: spaceElement; add: spaceElement; add: spaceElement. index _ index + 1].
	(self done not and: [(text at: index) = $.])
		ifTrue: [elements add: spaceElement; add: spaceElement; add: spaceElement; add: spaceElement. index _ index + 1].
	^ super skipBlanks! !

!TextToVoiceElementsTranslator methodsFor: 'all' stamp: 'len 4/13/98 19:18'!
spaceElement: aVoiceElement
	spaceElement _ aVoiceElement! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TextToVoiceElementsTranslator class
	instanceVariableNames: ''!

!TextToVoiceElementsTranslator class methodsFor: 'examples' stamp: 'len 4/13/98 19:26'!
english
	^ super english
		phonemesToElementsDictionary: VoiceElement englishPhonemesToElementsDictionary;
		endElement: VoiceElement end;
		spaceElement: VoiceElement space! !


Object subclass: #VoiceElement
	instanceVariableNames: 'rank duration maxDuration stress fn f1 f2 f3 b1 b2 b3 an a1 a2 a3 a4 a5 a6 ab av avc asp af '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
a1: aNumber
	a1 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
a2: aNumber
	a2 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
a3: aNumber
	a3 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
a4: aNumber
	a4 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
a5: aNumber
	a5 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
a6: aNumber
	a6 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
ab: aNumber
	ab _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
af: aNumber
	af _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:11'!
an: aNumber
	an _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
asp: aNumber
	asp _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
av: aNumber
	av _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:03'!
avc: aNumber
	avc _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:02'!
b1: aNumber
	b1 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:02'!
b2: aNumber
	b2 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:02'!
b3: aNumber
	b3 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/12/98 23:58'!
duration: aNumber
	duration _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:02'!
f1: aNumber
	f1 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:02'!
f2: aNumber
	f2 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:02'!
f3: aNumber
	f3 _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:02'!
fn: aNumber
	fn _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/13/98 00:55'!
maxDuration: aNumber
	maxDuration _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/12/98 23:58'!
rank: aNumber
	rank _ aNumber! !

!VoiceElement methodsFor: 'accessing-private' stamp: 'len 4/12/98 23:58'!
stress: aNumber
	stress _ aNumber! !


!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
a1
	^ a1! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
a2
	^ a2! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
a3
	^ a3! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
a4
	^ a4! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
a5
	^ a5! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
a6
	^ a6! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
ab
	^ ab! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
af
	^ af! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:11'!
an
	^ an! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:05'!
asp
	^ asp! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:05'!
av
	^ av! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:05'!
avc
	^ avc! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
b1
	^ b1! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
b2
	^ b2! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
b3
	^ b3! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/12/98 23:57'!
duration
	^ duration! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
f1
	^ f1! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
f2
	^ f2! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:04'!
f3
	^ f3! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/12/98 23:57'!
fn
	^ fn! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 5/1/98 20:52'!
frameFromParameters: anArray
	| adjust |
	adjust _ 14.
	^ self defaultFrame
		fnzhz: (anArray at: 1);
		f1hz: (anArray at: 2);
		f2hz: (anArray at: 3);
		f3hz: (anArray at: 4);
		b1hz: (anArray at: 5); b1phz: (anArray at: 5);
		b2hz: (anArray at: 6); b2phz: (anArray at: 6);
		b3hz: (anArray at: 7); b3phz: (anArray at: 7);
		anp: adjust + (anArray at: 8);
		a1: adjust + (anArray at: 9);
		a2: adjust + (anArray at: 10);
		a3: adjust + (anArray at: 11);
		a4: adjust + (anArray at: 12);
		a5: adjust + (anArray at: 13);
		a6: adjust + (anArray at: 14);
		ab: (anArray at: 15);
		avdb: (anArray at: 16);
		avpdb: (anArray at: 16);
		aturb: (anArray at: 17);
		asp: (anArray at: 18);
		af: (anArray at: 19)! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/13/98 00:55'!
maxDuration
	^ maxDuration! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/11/98 21:16'!
parameters
	"Answer an array of the parameters of the receiver."
	^ (Array new: 19)
		at: 1 put: fn;
		at: 2 put: f1;
		at: 3 put: f2;
		at: 4 put: f3;
		at: 5 put: b1;
		at: 6 put: b2;
		at: 7 put: b3;
		at: 8 put: an;
		at: 9 put: a1;
		at: 10 put: a2;
		at: 11 put: a3;
		at: 12 put: a4;
		at: 13 put: a5;
		at: 14 put: a6;
		at: 15 put: ab;
		at: 16 put: av;
		at: 17 put: avc;
		at: 18 put: asp;
		at: 19 put: af;
		yourself! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/12/98 23:58'!
rank
	^ rank! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 5/4/98 02:12'!
slopesWith: aVoiceElement
	| dominant dominated dominantParams dominatedParams dominantParam dominatedParam time value |
	dominant _ (self dominates: aVoiceElement) ifTrue: [self] ifFalse: [aVoiceElement].
	dominated _ dominant == self ifTrue: [aVoiceElement] ifFalse: [self].
	dominantParams _ dominant parameters.
	dominatedParams _ dominated parameters.
	^ (1 to: dominantParams size) collect: [ :each |
		dominantParam _ dominantParams at: each.
		dominatedParam _ dominatedParams at: each.
		time _ dominant == self
				ifTrue: [dominantParam internal]
				ifFalse: [dominantParam external].
		value _ time > 0
				ifTrue: [dominantParam proportion * dominatedParam steady * 0.01 + dominantParam fixed]
				ifFalse: [dominatedParam steady].
		time @ value]! !

!VoiceElement methodsFor: 'accessing' stamp: 'len 4/12/98 23:57'!
stress
	^ stress! !


!VoiceElement methodsFor: 'testing' stamp: 'len 4/12/98 23:59'!
dominates: aVoiceElement
	^ self rank > aVoiceElement rank! !


!VoiceElement methodsFor: 'private' stamp: 'len 4/13/98 01:19'!
defaultFrame
	^ VoiceFrame new
		f0hz10: 1330;
		avdb: 60;
		f1hz: 500;
		b1hz: 60;
		f2hz: 1500;
		b2hz: 90;
		f3hz: 2800;
		b3hz: 150;
		f4hz: 3250;
		b4hz: 200;
		f5hz: 3700;
		b5hz: 200;
		f6hz: 4990;
		b6hz: 500;
		fnzhz: 270;
		bnzhz: 100;
		fnphz: 270;
		bnphz: 100;
		asp: 0;
		kopen: 30;
		aturb: 0;
		tltdb: 10;
		af: 0;
		kskew: 0;
		a1: 0;
		b1phz: 80;
		a2: 0;
		b2phz: 200;
		a3: 0;
		b3phz: 350;
		a4: 0;
		b4phz: 500;
		a5: 0;
		b5phz: 600;
		a6: 0;
		b6phz: 800;
		anp: 0;
		ab: 0;
		avpdb: 0;
		gain0: 62! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

VoiceElement class
	instanceVariableNames: ''!

!VoiceElement class methodsFor: 'instance creation' stamp: 'len 5/1/98 20:30'!
fromArray: anArray
	^ self new
		rank: (anArray at: 1);
		maxDuration: (anArray at: 2);
		duration: (anArray at: 3);
		fn: (VoiceElementParameter fromArray: (anArray at: 4));
		f1: (VoiceElementParameter fromArray: (anArray at: 5));
		f2: (VoiceElementParameter fromArray: (anArray at: 6));
		f3: (VoiceElementParameter fromArray: (anArray at: 7));
		b1: (VoiceElementParameter fromArray: (anArray at: 8));
		b2: (VoiceElementParameter fromArray: (anArray at: 9));
		b3: (VoiceElementParameter fromArray: (anArray at: 10));
		an: (VoiceElementParameter fromArray: (anArray at: 11));
		a1: (VoiceElementParameter fromArray: (anArray at: 12));
		a2: (VoiceElementParameter fromArray: (anArray at: 13));
		a3: (VoiceElementParameter fromArray: (anArray at: 14));
		a4: (VoiceElementParameter fromArray: (anArray at: 15));
		a5: (VoiceElementParameter fromArray: (anArray at: 16));
		a6: (VoiceElementParameter fromArray: (anArray at: 17));
		ab: (VoiceElementParameter fromArray: (anArray at: 18));
		av: (VoiceElementParameter fromArray: (anArray at: 19));
		avc: (VoiceElementParameter fromArray: (anArray at: 20));
		asp: (VoiceElementParameter fromArray: (anArray at: 21));
		af: (VoiceElementParameter fromArray: (anArray at: 22))! !


!VoiceElement class methodsFor: 'examples' stamp: 'len 4/13/98 16:21'!
defaultPhonemesToElementsDictionary
	"Answer a Dictionary whose keys are phonemes and whose
	values are collections of corresponding elements."

	^ self englishPhonemesToElementsDictionary! !

!VoiceElement class methodsFor: 'examples' stamp: 'len 5/2/98 19:31'!
englishPhonemesToElementsDictionary
	"Answer a Dictionary whose keys are phonemes and whose
	values are collections of corresponding elements."

	| answer phonemes |
	phonemes _ Phoneme english.
	answer _ Dictionary new.
	#(($p	(p py pz))
	($t	(t ty tz))
	($k	(k ky kz))
	($b	(b by bz))
	($d	(d dy dz))
	($g	(g gy gz))
	($m	(m))
	($n	(n))
	($N	(ng))
	($f	(f))
	($T	(th))
	($s	(s))
	($S	(sh "ci"))
"	($x	(x))		this is not a phoneme"
	($h	(h))
	($v	(v qq v))
	($Z	(qq jy))
	($D	(dh qq di))
	($z	(z zz zh))
	($t	(ch))
	($l	(l "ll"))
	($R	(rx))
	($r	(r))
	($w	(w))
	($j	(y))
	($I	(i "ia"))
	($e	(e "ai air"))
	($&	(aa))
	($V	(u))
	($0	(o))
	($U	("oo" ov "oor"))
	($@	("a" oa "ib"))
	($i	(ee))
	($3	(er))
	($A	(ar))
	($O	(aw "or"))
	($u	(uu))
	($a	(ie "ou"))
	($o	(oi))) do: [ :each |
		answer at: (phonemes detect: [ :one | one ipa = each first])
		put: (each last collect: [ :selector | self perform: selector])].
	^ answer! !


!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 00:55'!
a
	^ self fromArray:
		"rank	dur		max dur"
		#(2		4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		230		50		4		4)	"f1"
		(1480	710		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(50.75	24.5		50		4		4)	"a2"
		(33.25	17.5		50		4		4)	"a3"
		(26.25	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:59'!
aa
	^ self fromArray:
		"rank	dur		max dur"
		#(2		10		5
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(790		410		50		4		4)	"f1"
		(1780	950		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(130		65		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(42		21		50		4		4)	"a2"
		(38.5	17.5		50		4		4)	"a3"
		(31.5	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:22'!
ai
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(640		290		50		5		5)	"f1"
		(1600	830		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(45.5	21		50		5		5)	"a2"
		(35		17.5		50		5		5)	"a3"
		(29.75	14		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:45'!
air
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(640		350		50		5		5)	"f1"
		(2020	1070		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(42		21		50		5		5)	"a2"
		(38.5	17.5		50		5		5)	"a3"
		(31.5	14		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:15'!
ar
	^ self fromArray:
		"rank	dur		max dur"
		#(2		15		15
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(790		410		50		4		4)	"f1"
		(880		470		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(49		24.5		50		4		4)	"a2"
		(29.75	14		50		4		4)	"a3"
		(22.75	10.5		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:18'!
aw
	^ self fromArray:
		"rank	dur		max dur"
		#(2		16		10
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		230		50		4		4)	"f1"
		(820		470		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(45.5	21		50		4		4)	"a2"
		(22.75	10.5		50		4		4)	"a3"
		(17.5	7		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:04'!
b
	^ self fromArray:
		"rank	dur		max dur"
		#(26	12		12
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		2)	"f1"
		(760		350		50		2		2)	"f2"
		(2500	0		100		0		2)	"f3"
		(60		30		50		2		2)	"b1"
		(90		45		50		2		2)	"b2"
		(150		0		100		0		2)	"b3"
		(-30		0		100		0		0)	"an"
		(24.5	0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:06'!
by
	^ self fromArray:
		"rank	dur		max dur"
		#(29	1		1
		"steady	fixed	prop	extern	intern"
		(270		135		100		0		0)	"fn"
		(190		0		100		0		0)	"f1"
		(760		0		100		0		0)	"f2"
		(2500	0		100		0		0)	"f3"
		(60		0		100		0		0)	"b1"
		(90		0		100		0		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(24.5	0		100		0		0)	"a1"
		(49		0		100		0		0)	"a2"
		(43.25	0		100		0		0)	"a3"
		(38.5	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:15'!
bz
	^ self fromArray:
		"rank	dur		max dur"
		#(26	0		0
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		0)	"f1"
		(760		350		50		2		0)	"f2"
		(2500	0		100		0		0)	"f3"
		(60		30		50		2		0)	"b1"
		(90		45		50		2		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:24'!
ch
	^ self fromArray:
		"rank	dur		max dur"
		#(23	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		2)	"f1"
		(1780	950		50		2		2)	"f2"
		(2680	2680	0		2		2)	"f3"
		(60		30		50		2		2)	"b1"
		(90		45		50		2		2)	"b2"
		(150		150		0		2		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:26'!
ci
	^ self fromArray:
		"rank	dur		max dur"
		#(18		8		8
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(400		170		50		3		2)	"f1"
		(2020	1190		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(31.5	0		100		0		0)	"a2"
		(42		0		100		0		0)	"a3"
		(31.5	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:17'!
d
	^ self fromArray:
		"rank	dur		max dur"
		#(26	8		8
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		2)	"f1"
		(1780	950		50		2		2)	"f2"
		(2680	2680	0		2		2)	"f3"
		(60		30		50		2		2)	"b1"
		(90		45		50		2		2)	"b2"
		(150		150		0		2		2)	"b3"
		(-30		0		100		0		0)	"an"
		(31.5	0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:22'!
dh
	^ self fromArray:
		"rank	dur		max dur"
		#(20	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		170		50		3		2)	"f1"
		(1600	1190		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(31.5	0		100		0		0)	"a2"
		(26.25	0		100		0		0)	"a3"
		(28		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(54		27		100		0		0)	"ab"
		(36		18		50		0		0)	"av"
		(54		27		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:13'!
di
	^ self fromArray:
		"rank	dur		max dur"
		#(20	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		170		50		3		2)	"f1"
		(1600	1190		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(31.5	0		100		0		0)	"a2"
		(26.25	0		100		0		0)	"a3"
		(28		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:21'!
dy
	^ self fromArray:
		"rank	dur		max dur"
		#(29	1		1
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		0		100		0		0)	"f1"
		(1780	0		100		0		0)	"f2"
		(2680	0		100		0		0)	"f3"
		(60		0		100		0		0)	"b1"
		(90		0		100		0		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(38.5	0		100		0		0)	"a1"
		(38.5	0		100		0		0)	"a2"
		(35		0		100		0		0)	"a3"
		(45.5	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:30'!
dz
	^ self fromArray:
		"rank	dur		max dur"
		#(26	1		1
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		0)	"f1"
		(1780	950		50		2		0)	"f2"
		(2680	2680	0		2		0)	"f3"
		(60		30		50		2		0)	"b1"
		(90		45		50		2		0)	"b2"
		(150		150		0		2		0)	"b3"
		(-30		0		100		0		0)	"an"
		(38.5	0		100		0		0)	"a1"
		(28		0		100		0		0)	"a2"
		(24.5	0		100		0		0)	"a3"
		(35		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:57'!
e
	^ self fromArray:
		"rank	dur		max dur"
		#(2		8		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(640		350		50		4		4)	"f1"
		(2080	1070		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(42		21		50		4		4)	"a2"
		(38.5	17.5		50		4		4)	"a3"
		(31.5	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:11'!
ee
	^ self fromArray:
		"rank	dur		max dur"
		#(2		11		7
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(250		110		50		4		4)	"f1"
		(2320	1190		50		4		4)	"f2"
		(3200	1580		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(33.25	17.5		50		4		4)	"a2"
		(36.75	17.5		50		4		4)	"a3"
		(31.5	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 00:55'!
end
	^ self fromArray:
		#(31		5		5
		(270		135		50		3		3)
		(490		0		100		0		0)
		(1480	0		100		0		0)
		(2500	0		100		0		0)
		(60		0		100		0		0)
		(90		0		100		0		0)
		(150		0		100		0		0)
		(-30		-10.5	100		3		0)
		(-30		-10.5	100		3		0)
		(-30		-10.5	100		3		0)
		(-30		-10.5	100		3		0)
		(-30		-10.5	100		3		0)
		(-30		0		100		3		0)
		(-30		0		100		3		0)
		(-30		0		100		3		0)
		(0		0		50		0		0)
		(0		0		50		0		0)
		(0		0		50		0		0)
		(0		0		50		0		0))
! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:13'!
er
	^ self fromArray:
		"rank	dur		max dur"
		#(2		16		16
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(580		290		50		4		4)	"f1"
		(1420	710		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(45.5	21		50		4		4)	"a2"
		(33.25	17.5		50		4		4)	"a3"
		(26.25	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:54'!
f
	^ self fromArray:
		"rank	dur		max dur"
		#(18		12		12
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(400		170		50		3		2)	"f1"
		(1420	350		50		3		2)	"f2"
		(2560	980		50		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		75		50		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(0		0		100		0		0)	"a2"
		(0		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(54		27		50		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(32		16		50		0		0)	"asp"
		(54		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:31'!
g
	^ self fromArray:
		"rank	dur		max dur"
		#(26	12		12
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		3		3)	"f1"
		(1480	1550		50		3		3)	"f2"
		(2620	1580		50		3		3)	"f3"
		(60		30		50		3		3)	"b1"
		(90		45		50		3		3)	"b2"
		(150		75		50		3		3)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:33'!
gy
	^ self fromArray:
		"rank	dur		max dur"
		#(29	1		1
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		0		100		0		0)	"f1"
		(1480	0		100		0		0)	"f2"
		(2620	0		100		0		0)	"f3"
		(60		0		100		0		0)	"b1"
		(90		0		100		0		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(35		0		100		0		0)	"a1"
		(45.5	0		100		0		0)	"a2"
		(40.25	0		100		0		0)	"a3"
		(24.5	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:35'!
gz
	^ self fromArray:
		"rank	dur		max dur"
		#(26	2		2
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		3		2)	"f1"
		(1480	1550		50		3		2)	"f2"
		(2620	1580		50		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		75		50		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(35		0		100		0		0)	"a1"
		(35		0		100		0		0)	"a2"
		(29.75	0		100		0		0)	"a3"
		(14		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:13'!
h
	^ self fromArray:
		"rank	dur		max dur"
		#(9		10		10
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		0		100		0		7)	"f1"
		(1480	0		100		0		7)	"f2"
		(2500	0		100		0		7)	"f3"
		(60		0		100		0		7)	"b1"
		(90		0		100		0		7)	"b2"
		(150		0		100		0		7)	"b3"
		(-30		0		100		0		7)	"an"
		(35		-14		100		0		7)	"a1"
		(36.75	-14		100		0		7)	"a2"
		(26.25	-7		100		0		7)	"a3"
		(22.75	-3.5		100		0		7)	"a4"
		(-30		0		100		0		7)	"a5"
		(-30		0		100		0		7)	"a6"
		(-30		0		100		0		7)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:54'!
i
	^ self fromArray:
		"rank	dur		max dur"
		#(2		8		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(400		170		50		4		4)	"f1"
		(2080	1070		50		4		4)	"f2"
		(2560	1340		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(36.75	17.5		50		4		4)	"a2"
		(35		17.5		50		4		4)	"a3"
		(29.75	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:40'!
ia
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(310		170		50		5		5)	"f1"
		(2200	1070		50		5		5)	"f2"
		(2920	1460		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(35		17.5		50		5		5)	"a2"
		(36.75	17.5		50		5		5)	"a3"
		(31.5	14		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:42'!
ib
	^ self fromArray:
		"rank	dur		max dur"
		#(2		8		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		230		50		4		4)	"f1"
		(1480	710		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(50.75	24.5		50		4		4)	"a2"
		(33.25	17.5		50		4		4)	"a3"
		(26.25	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:24'!
ie
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(790		410		50		5		5)	"f1"
		(880		470		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(49		24.5		50		5		5)	"a2"
		(29.75	14		50		5		5)	"a3"
		(22.75	10.5		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:28'!
j
	^ self fromArray:
		"rank	dur		max dur"
		#(26	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		2)	"f1"
		(1780	950		50		2		2)	"f2"
		(2680	2680	0		2		2)	"f3"
		(60		30		50		2		2)	"b1"
		(90		45		50		2		2)	"b2"
		(150		150		0		2		2)	"b3"
		(-30		0		100		0		0)	"an"
		(31.5	0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:31'!
jy
	^ self fromArray:
		"rank	dur		max dur"
		#(20	3		3
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		170		50		3		2)	"f1"
		(2020	1190		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(26.25	0		100		0		0)	"a2"
		(36.75	0		100		0		0)	"a3"
		(26.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 01:57'!
k
	^ self fromArray:
		"rank	dur		max dur"
		#(23	8		8
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		3		3)	"f1"
		(1480	1550		50		3		3)	"f2"
		(2620	1580		50		3		3)	"f3"
		(60		30		50		3		3)	"b1"
		(90		45		50		3		3)	"b2"
		(150		75		50		3		3)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 01:58'!
ky
	^ self fromArray:
		"rank	dur		max dur"
		#(29	1		1
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		0		100		0		0)	"f1"
		(1480	0		100		0		0)	"f2"
		(2620	0		100		0		0)	"f3"
		(60		0		100		0		0)	"b1"
		(90		0		100		0		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(50.75	0		100		0		0)	"a2"
		(50.75	0		100		0		0)	"a3"
		(29.75	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:01'!
kz
	^ self fromArray:
		"rank	dur		max dur"
		#(23	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		3		3)	"f1"
		(1480	1550		50		3		3)	"f2"
		(2620	1580		50		3		3)	"f3"
		(60		30		50		3		3)	"b1"
		(90		45		50		3		3)	"b2"
		(150		75		50		3		3)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(40.25	0		100		0		0)	"a2"
		(40.25	0		100		0		0)	"a3"
		(19.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:34'!
l
	^ self fromArray:
		"rank	dur		max dur"
		#(11		8		8
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(460		230		50		6		0)	"f1"
		(1480	710		50		6		0)	"f2"
		(2500	1220		50		6		0)	"f3"
		(60		30		50		6		0)	"b1"
		(90		45		50		6		0)	"b2"
		(150		75		50		6		0)	"b3"
		(-30		0		100		0		0)	"an"
		(36.75	0		100		0		0)	"a1"
		(26.25	0		100		0		0)	"a2"
		(26.25	0		100		0		0)	"a3"
		(21		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:35'!
ll
	^ self fromArray:
		"rank	dur		max dur"
		#(11		8		8
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(460		230		50		6		0)	"f1"
		(940		470		50		6		0)	"f2"
		(2500	1220		50		6		0)	"f3"
		(60		30		50		6		0)	"b1"
		(90		45		50		6		0)	"b2"
		(150		75		50		6		0)	"b3"
		(-30		0		100		0		0)	"an"
		(36.75	0		100		0		0)	"a1"
		(26.25	0		100		0		0)	"a2"
		(26.25	0		100		0		0)	"a3"
		(21		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:41'!
m
	^ self fromArray:
		"rank	dur		max dur"
		#(15		8		8
		"steady	fixed	prop	extern	intern"
		(360		360		0		3		0)	"fn"
		(480		480		0		3		0)	"f1"
		(1000	350		50		3		0)	"f2"
		(2200	0		100		5		0)	"f3"
		(40		20		50		3		0)	"b1"
		(175		87		50		3		0)	"b2"
		(120		0		100		5		0)	"b3"
		(42		21		50		3		0)	"an"
		(26		-10		100		3		0)	"a1"
		(30		-10		100		3		0)	"a2"
		(33		-10		100		3		0)	"a3"
		(-30		-10		100		3		0)	"a4"
		(-30		0		100		3		0)	"a5"
		(-30		0		100		3		0)	"a6"
		(-30		0		100		3		0)	"ab"
		(62		31		50		2		0)	"av"
		(62		31		50		2		0)	"avc"
		(0		0		50		2		0)	"asp"
		(0		0		50		2		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:44'!
n
	^ self fromArray:
		"rank	dur		max dur"
		#(15		8		8
		"steady	fixed	prop	extern	intern"
		(450		450		0		3		0)	"fn"
		(480		480		0		3		0)	"f1"
		(1780	950		50		3		3)	"f2"
		(2680	2680	0		3		0)	"f3"
		(40		20		50		3		0)	"b1"
		(300		150		50		3		3)	"b2"
		(260		130		50		3		0)	"b3"
		(42		21		50		3		0)	"an"
		(35		-10		100		3		0)	"a1"
		(35		-10		100		3		0)	"a2"
		(35		-10		100		3		0)	"a3"
		(20		-10		100		3		0)	"a4"
		(-30		0		100		3		0)	"a5"
		(-30		0		100		3		0)	"a6"
		(-30		0		100		3		0)	"ab"
		(62		31		50		2		0)	"av"
		(62		31		50		2		0)	"avc"
		(0		0		50		2		0)	"asp"
		(0		0		50		2		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 02:48'!
ng
	^ self fromArray:
		"rank	dur		max dur"
		#(15		8		8
		"steady	fixed	prop	extern	intern"
		(360		360		0		3		0)	"fn"
		(480		480		0		3		0)	"f1"
		(820		1550		50		5		0)	"f2"
		(2800	1580		50		3		0)	"f3"
		(160		80		0		5		0)	"b1"
		(150		75		50		5		0)	"b2"
		(120		50		50		3		0)	"b3"
		(42		21		50		3		0)	"an"
		(20		0		100		3		0)	"a1"
		(30		0		100		3		0)	"a2"
		(35		0		100		3		0)	"a3"
		(0		0		100		3		0)	"a4"
		(-30		0		100		3		0)	"a5"
		(-30		0		100		3		0)	"a6"
		(-30		0		100		3		0)	"ab"
		(52		26		50		2		0)	"av"
		(56		28		50		2		0)	"avc"
		(0		0		50		2		0)	"asp"
		(0		0		50		2		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:03'!
o
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(610		290		50		4		4)	"f1"
		(880		470		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(47.25	24.5		50		4		4)	"a2"
		(31.5	10.5		50		4		4)	"a3"
		(15.75	7		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:33'!
oa
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		230		50		5		5)	"f1"
		(1480	710		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(50.75	24.5		50		5		5)	"a2"
		(33.25	17.5		50		5		5)	"a3"
		(26.25	14		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:26'!
oi
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		230		50		5		5)	"f1"
		(820		350		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(45.5	21		50		5		5)	"a2"
		(22.75	10.5		50		5		5)	"a3"
		(17.5	7		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:05'!
oo
	^ self fromArray:
		"rank	dur		max dur"
		#(2		6		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(370		170		50		4		4)	"f1"
		(1000	470		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(42		21		50		4		4)	"a2"
		(28		14		50		4		4)	"a3"
		(22.75	10.5		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:47'!
oor
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(370		170		50		5		5)	"f1"
		(1000	470		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(42		21		50		5		5)	"a2"
		(28		14		50		5		5)	"a3"
		(22.75	7		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:49'!
or
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		230		50		5		5)	"f1"
		(820		470		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(45.5	21		50		5		5)	"a2"
		(22.75	10.5		50		5		5)	"a3"
		(17.5	7		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:28'!
ou
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(790		410		50		5		5)	"f1"
		(1300	590		50		5		5)	"f2"
		(2500	1220		50		5		5)	"f3"
		(60		30		50		5		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(50.75	24.5		50		5		5)	"a1"
		(47.25	24.5		50		5		5)	"a2"
		(35		17.5		50		5		5)	"a3"
		(28		14		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:31'!
ov
	^ self fromArray:
		"rank	dur		max dur"
		#(2		8		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(370		170		50		4		4)	"f1"
		(1000	470		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(42		21		50		4		4)	"a2"
		(28		14		50		4		4)	"a3"
		(22.75	10.5		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 01:51'!
p
	^ self fromArray:
		"rank	dur		max dur"
		#(23	8		8
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		2)	"f1"
		(760		350		50		2		2)	"f2"
		(2500	0		100		0		2)	"f3"
		(60		30		50		2		2)	"b1"
		(90		45		50		2		2)	"b2"
		(150		0		100		0		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 00:56'!
py
	^ self fromArray:
		"rank	dur		max dur"
		#(29	1		1
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		0		100		0		0)	"f1"
		(760		0		100		0		0)	"f2"
		(2500	0		100		0		0)	"f3"
		(60		0		100		0		0)	"b1"
		(90		0		100		0		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(24.5	0		100		0		0)	"a1"
		(49		0		100		0		0)	"a2"
		(23.75	0		100		0		0)	"a3"
		(38.5	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 00:56'!
pz
	^ self fromArray:
		"rank	dur		max dur"
		#(23	2		2
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		2)	"f1"
		(760		350		50		2		2)	"f2"
		(2500	0		100		2		2)	"f3"
		(60		30		50		2		2)	"b1"
		(90		45		50		2		2)	"b2"
		(150		0		100		2		2)	"b3"
		(-30		0		100		0		0)	"an"
		(24.5	0		100		0		0)	"a1"
		(38.5	0		100		0		0)	"a2"
		(33.25	0		100		0		0)	"a3"
		(28		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:19'!
qq
	^ self fromArray:
		"rank	dur		max dur"
		#(30	0		0
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		0		100		0		0)	"f1"
		(1420	0		100		0		0)	"f2"
		(2560	0		100		0		0)	"f3"
		(60		0		100		0		0)	"b1"
		(90		0		100		0		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(40.25	0		100		0		0)	"a2"
		(36.75	0		100		0		0)	"a3"
		(33.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:44'!
r
	^ self fromArray:
		"rank	dur		max dur"
		#(10		11		11
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		0		100		0		5)	"f1"
		(1180	590		50		5		5)	"f2"
		(1600	740		50		5		5)	"f3"
		(60		0		100		0		5)	"b1"
		(90		45		50		5		5)	"b2"
		(150		75		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(42		21		50		5		5)	"a1"
		(35		17.5		50		5		5)	"a2"
		(35		17.5		50		5		5)	"a3"
		(-30		0		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:39'!
rx
	^ self fromArray:
		"rank	dur		max dur"
		#(10		10		10
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(490		0		100		0		5)	"f1"
		(1180	0		100		0		5)	"f2"
		(1600	1600		0		5		5)	"f3"
		(60		30		50		0		5)	"b1"
		(90		45		50		5		5)	"b2"
		(70		35		50		5		5)	"b3"
		(-30		0		100		5		5)	"an"
		(42		21		50		5		5)	"a1"
		(35		17.5		50		5		5)	"a2"
		(35		17.5		50		5		5)	"a3"
		(-30		0		50		5		5)	"a4"
		(-30		-15		50		5		5)	"a5"
		(-30		-15		50		5		5)	"a6"
		(-30		-15		50		5		5)	"ab"
		(50		25		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:08'!
s
	^ self fromArray:
		"rank	dur		max dur"
		#(18		12		12
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(400		170		50		3		2)	"f1"
		(1720	950		50		3		2)	"f2"
		(2620	0		100		3		2)	"f3"
		(200		100		50		3		2)	"b1"
		(96		48		50		3		2)	"b2"
		(220		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(28		0		100		0		0)	"a2"
		(28		0		100		0		0)	"a3"
		(40.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(32		16		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:08'!
sh
	^ self fromArray:
		"rank	dur		max dur"
		#(18		12		12
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(400		170		50		3		2)	"f1"
		(2200	1190		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(31.5	0		100		0		0)	"a2"
		(42		0		100		0		0)	"a3"
		(31.5	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 00:56'!
space
	^ self fromArray:
		#(29	6		6
		(270		135		50		3		3)	"fn"
		(490		0		100		3		3)	"f1"
		(1480	0		100		3		3)	"f2"
		(2500	0		100		3		3)	"f3"
		(60		0		100		3		3)	"b1"
		(90		0		100		3		3)	"b2"
		(150		0		100		3		3)	"b3"
		(-30		-10.5	100		3		0)	"an"
		(-30		-10.5	100		3		0)	"a1"
		(-30		-10.5	100		3		0)	"a2"
		(-30		-10.5	100		3		0)	"a3"
		(-30		-10.5	100		3		0)	"a4"
		(-30		0		100		3		0)	"a5"
		(-30		0		100		3		0)	"a6"
		(-30		0		100		3		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"
! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 01:48'!
t
	^ self fromArray:
		"rank	dur		max dur"
		#(23	6		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		2)	"f1"
		(1780	950		50		2		2)	"f2"
		(2680	2680	0		0		2)	"f3"
		(60		30		50		2		2)	"b1"
		(90		45		50		2		2)	"b2"
		(150		150		0		0		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(-30		0		100		0		0)	"a3"
		(-30		0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:09'!
th
	^ self fromArray:
		"rank	dur		max dur"
		#(18		15		15
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(400		170		50		3		2)	"f1"
		(1780	1190		50		3		2)	"f2"
		(2680	2680	0		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		150		0		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(26.25	0		100		0		0)	"a2"
		(28		0		100		0		0)	"a3"
		(22.75	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 01:51'!
ty
	^ self fromArray:
		"rank	dur		max dur"
		#(29	1		1
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		0		100		0		0)	"f1"
		(1780	0		100		0		0)	"f2"
		(2680	0		100		0		0)	"f3"
		(60		0		100		0		0)	"b1"
		(90		0		100		0		0)	"b2"
		(150		0		100		0		0)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(38.5	0		100		0		0)	"a3"
		(50.75	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 01:54'!
tz
	^ self fromArray:
		"rank	dur		max dur"
		#(23	2		2
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		2		1)	"f1"
		(1780	950		50		2		1)	"f2"
		(2680	2680	0		2		0)	"f3"
		(60		30		50		2		1)	"b1"
		(90		45		50		2		1)	"b2"
		(150		150		0		2		0)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(-30		0		100		0		0)	"a2"
		(28		0		100		0		0)	"a3"
		(40.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:01'!
u
	^ self fromArray:
		"rank	dur		max dur"
		#(2		9		6
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(700		350		50		4		4)	"f1"
		(1360	710		50		4		4)	"f2"
		(2500	1220		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(43.75	21		50		4		4)	"a2"
		(31.5	14		50		4		4)	"a3"
		(24.5	10.5		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 15:19'!
uu
	^ self fromArray:
		"rank	dur		max dur"
		#(2		14		9
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(250		110		50		4		4)	"f1"
		(880		470		50		4		4)	"f2"
		(2200	1100		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(38.5	17.5		50		4		4)	"a2"
		(17.5	7		50		4		4)	"a3"
		(10.5	3.5		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:16'!
v
	^ self fromArray:
		"rank	dur		max dur"
		#(20	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		170		50		3		2)	"f1"
		(1420	350		50		3		2)	"f2"
		(2560	980		50		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		75		50		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(40.25	0		100		0		0)	"a2"
		(36.75	0		100		0		0)	"a3"
		(33.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:48'!
w
	^ self fromArray:
		"rank	dur		max dur"
		#(10		8		8
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		50		50		4		4)	"f1"
		(760		350		50		4		4)	"f2"
		(2020	980		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(43.75	21		50		4		4)	"a1"
		(28		14		50		4		4)	"a2"
		(21		10.5		50		4		4)	"a3"
		(-30		0		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 03:08'!
x
	^ self fromArray:
		"rank	dur		max dur"
		#(18		12		12
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(190		110		50		3		3)	"f1"
		(1480	1550		50		3		3)	"f2"
		(2620	1580		50		3		3)	"f3"
		(60		30		50		3		3)	"b1"
		(90		45		50		3		3)	"b2"
		(150		75		50		3		3)	"b3"
		(-30		0		100		0		0)	"an"
		(-30		0		100		0		0)	"a1"
		(40.25	0		100		0		0)	"a2"
		(40.25	0		100		0		0)	"a3"
		(19.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(0		0		50		0		0)	"av"
		(0		0		50		0		0)	"avc"
		(60		30		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:51'!
y
	^ self fromArray:
		"rank	dur		max dur"
		#(10		7		7
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(250		110		50		4		4)	"f1"
		(2500	1190		50		4		4)	"f2"
		(2980	1460		50		4		4)	"f3"
		(60		30		50		4		4)	"b1"
		(90		45		50		4		4)	"b2"
		(150		75		50		4		4)	"b3"
		(-30		0		100		4		4)	"an"
		(50.75	24.5		50		4		4)	"a1"
		(33.25	17.5		50		4		4)	"a2"
		(38.5	17.5		50		4		4)	"a3"
		(31.5	14		50		4		4)	"a4"
		(-30		-15		50		4		4)	"a5"
		(-30		-15		50		4		4)	"a6"
		(-30		-15		50		4		4)	"ab"
		(62		31		50		0		0)	"av"
		(16		8		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:16'!
z
	^ self fromArray:
		"rank	dur		max dur"
		#(20	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		170		50		3		2)	"f1"
		(1720	950		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(24.5	0		100		0		0)	"a2"
		(24.5	0		100		0		0)	"a3"
		(36.75	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(40		20		50		0		0)	"av"
		(54		27		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(60		30		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:21'!
zh
	^ self fromArray:
		"rank	dur		max dur"
		#(20	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		170		50		3		2)	"f1"
		(2020	1190		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(26.25	0		100		0		0)	"a2"
		(36.75	0		100		0		0)	"a3"
		(26.25	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !

!VoiceElement class methodsFor: 'examples-default elements' stamp: 'len 4/13/98 14:18'!
zz
	^ self fromArray:
		"rank	dur		max dur"
		#(20	4		4
		"steady	fixed	prop	extern	intern"
		(270		135		50		0		0)	"fn"
		(280		170		50		3		2)	"f1"
		(1720	950		50		3		2)	"f2"
		(2560	0		100		3		2)	"f3"
		(60		30		50		3		2)	"b1"
		(90		45		50		3		2)	"b2"
		(150		0		100		3		2)	"b3"
		(-30		0		100		0		0)	"an"
		(29.75	0		100		0		0)	"a1"
		(24.5	0		100		0		0)	"a2"
		(24.5	0		100		0		0)	"a3"
		(36.75	0		100		0		0)	"a4"
		(-30		0		100		0		0)	"a5"
		(-30		0		100		0		0)	"a6"
		(-30		0		100		0		0)	"ab"
		(62		31		50		0		0)	"av"
		(62		31		50		0		0)	"avc"
		(0		0		50		0		0)	"asp"
		(0		0		50		0		0))	"af"! !


Object subclass: #VoiceElementParameter
	instanceVariableNames: 'steady fixed proportion internal external '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:37'!
external
	^ external! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:36'!
external: aNumber
	external _ aNumber! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/13/98 01:01'!
fixed
	^ fixed! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:36'!
fixed: aNumber
	fixed _ aNumber! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:37'!
internal
	^ internal! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:36'!
internal: aNumber
	internal _ aNumber! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:37'!
proportion
	^ proportion! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:37'!
proportion: aNumber
	proportion _ aNumber! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:37'!
steady
	^ steady! !

!VoiceElementParameter methodsFor: 'accessing' stamp: 'len 4/11/98 21:36'!
steady: aNumber
	steady _ aNumber! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

VoiceElementParameter class
	instanceVariableNames: ''!

!VoiceElementParameter class methodsFor: 'instance creation' stamp: 'len 4/13/98 00:13'!
fromArray: anArray
	^ self new
		steady: (anArray at: 1);
		fixed: (anArray at: 2);
		proportion: (anArray at: 3);
		external: (anArray at: 4);
		internal: (anArray at: 5)! !


Object subclass: #VoiceFrame
	instanceVariableNames: 'f0hz10 avdb f1hz b1hz f2hz b2hz f3hz b3hz f4hz b4hz f5hz b5hz f6hz b6hz fnzhz bnzhz fnphz bnphz asp kopen aturb tltdb af kskew a1 b1phz a2 b2phz a3 b3phz a4 b4phz a5 b5phz a6 b6phz anp ab avpdb gain0 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!
!VoiceFrame commentStamp: 'len 5/5/98 01:10' prior: 0!
My instances are frames of parameters which are inputs for VoiceSynthesizers.!


!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
a1
	^ a1! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:54'!
a1: aNumber
	a1 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
a2
	^ a2! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:54'!
a2: aNumber
	a2 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
a3
	^ a3! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
a3: aNumber
	a3 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
a4
	^ a4! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
a4: aNumber
	a4 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
a5
	^ a5! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
a5: aNumber
	a5 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
a6
	^ a6! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
a6: aNumber
	a6 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
ab
	^ ab! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
ab: aNumber
	ab _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
af
	^ af! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:00'!
af: aNumber
	af _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
anp
	^ anp! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
anp: aNumber
	anp _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
asp
	^ asp! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:02'!
asp: aNumber
	asp _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
aturb
	^ aturb! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:00'!
aturb: aNumber
	aturb _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
avdb
	^ avdb! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:54'!
avdb: aNumber
	avdb _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
avpdb
	^ avpdb! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
avpdb: aNumber
	avpdb _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
b1hz
	^ b1hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:53'!
b1hz: aNumber
	b1hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:01'!
b1phz
	^ b1phz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:53'!
b1phz: aNumber
	b1phz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
b2hz
	^ b2hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:53'!
b2hz: aNumber
	b2hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:01'!
b2phz
	^ b2phz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:53'!
b2phz: aNumber
	b2phz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
b3hz
	^ b3hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:53'!
b3hz: aNumber
	b3hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:01'!
b3phz
	^ b3phz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:53'!
b3phz: aNumber
	b3phz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
b4hz
	^ b4hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
b4hz: aNumber
	b4hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:01'!
b4phz
	^ b4phz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:53'!
b4phz: aNumber
	b4phz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
b5hz
	^ b5hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
b5hz: aNumber
	b5hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:01'!
b5phz
	^ b5phz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:54'!
b5phz: aNumber
	b5phz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
b6hz
	^ b6hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
b6hz: aNumber
	b6hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:01'!
b6phz
	^ b6phz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:54'!
b6phz: aNumber
	b6phz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
bnphz
	^ bnphz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:30'!
bnphz: aNumber
	bnphz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:28'!
bnzhz
	^ bnzhz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:28'!
bnzhz: aNumber
	bnzhz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:56'!
f0hz10
	^ f0hz10! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
f0hz10: aNumber
	f0hz10 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
f1hz
	^ f1hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
f1hz: aNumber
	f1hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
f2hz
	^ f2hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
f2hz: aNumber
	f2hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
f3hz
	^ f3hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
f3hz: aNumber
	f3hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
f4hz
	^ f4hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
f4hz: aNumber
	f4hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:57'!
f5hz
	^ f5hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
f5hz: aNumber
	f5hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:58'!
f6hz
	^ f6hz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:52'!
f6hz: aNumber
	f6hz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:59'!
fnphz
	^ fnphz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:29'!
fnphz: aNumber
	fnphz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:29'!
fnzhz
	^ fnzhz! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:55'!
fnzhz: aNumber
	fnzhz _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:56'!
gain0
	^ gain0! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:56'!
gain0: aNumber
	gain0 _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:00'!
kopen
	^ kopen! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:00'!
kopen: aNumber
	kopen _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:00'!
kskew
	^ kskew! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:54'!
kskew: aNumber
	kskew _ aNumber! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 06:00'!
tltdb
	^ tltdb! !

!VoiceFrame methodsFor: 'accessing' stamp: 'len 4/8/98 05:56'!
tltdb: aNumber
	tltdb _ aNumber! !


!VoiceFrame methodsFor: 'printing' stamp: 'len 4/8/98 06:27'!
printOn: aStream
	| selectors |
	selectors _ #(f0hz10 avdb f1hz b1hz f2hz b2hz f3hz b3hz f4hz b4hz f5hz b5hz f6hz b6hz fnzhz bnzhz fnphz bnphz asp kopen aturb tltdb af kskew a1 b1phz a2 b2phz a3 b3phz a4 b4phz a5 b5phz a6 b6phz anp ab avpdb gain0).
	selectors do: [ :each | aStream print: (self perform: each); space]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

VoiceFrame class
	instanceVariableNames: ''!

!VoiceFrame class methodsFor: 'instance creation' stamp: 'len 5/1/98 03:48'!
allFromElementsFileNamed: fileName
	| file answer elem string |
	elem _ VoiceElement new.
	answer _ OrderedCollection new.
	file _ FileStream oldFileNamed: fileName.
	[file atEnd] whileFalse: [
		string _ file nextLine.
		string first = $# ifFalse: [
			answer add: (elem frameFromParameters: (string asArrayOfSubstrings collect: [ :each | each asNumber]))]].
	file close.
	^ answer! !

!VoiceFrame class methodsFor: 'instance creation' stamp: 'len 4/8/98 23:36'!
allFromFileNamed: fileName
	| file answer |
	answer _ OrderedCollection new.
	file _ FileStream oldFileNamed: fileName.
	[file atEnd] whileFalse: [answer add: (self fromStream: file)].
	file close.
	^ answer! !

!VoiceFrame class methodsFor: 'instance creation' stamp: 'len 4/13/98 01:03'!
fromArray: anArray
	| selectors answer |
	selectors _
		#(f0hz10: avdb: f1hz: b1hz: f2hz: b2hz: f3hz: b3hz: f4hz: b4hz: f5hz: b5hz: f6hz: b6hz:
		fnzhz: bnzhz: fnphz: bnphz: asp: kopen: aturb: tltdb: af: kskew:
		a1: b1phz: a2: b2phz: a3: b3phz: a4: b4phz: a5: b5phz: a6: b6phz: anp: ab: avpdb: gain0:).
	answer _ self new.
	1 to: 40 do: [ :each | answer perform: (selectors at: each) with: (anArray at: each)].
	^ answer
! !

!VoiceFrame class methodsFor: 'instance creation' stamp: 'len 4/8/98 23:36'!
fromStream: aStream
	^ self fromString: aStream nextLine! !

!VoiceFrame class methodsFor: 'instance creation' stamp: 'len 4/8/98 23:38'!
fromString: aString
	^ self fromArray: (aString asArrayOfSubstrings collect: [ :each | each asNumber])! !


Object subclass: #VoiceInterpolator
	instanceVariableNames: 'left current right stressStart stressEnd parametersStart parametersEnd filters f0hz '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech Synthesis'!
!VoiceInterpolator commentStamp: 'len 5/5/98 01:10' prior: 0!
My instances are employed in the soft join of VoiceElements to produce VoiceFrames, which are the input of VoiceSynthesizers. Intonation should be implemented here. Stress is not yet implemented, but will be soon.!


!VoiceInterpolator methodsFor: 'initialization' stamp: 'len 5/4/98 02:14'!
initialize
	stressStart _ 40 @ 0.0.
	stressEnd _ 40 @ 0.0.
	f0hz _ 1330! !

!VoiceInterpolator methodsFor: 'initialization' stamp: 'len 4/12/98 20:01'!
initializeFilters
	| frac |
	frac _ 1.0.
	filters _ left parameters collect: [ :each | SimpleVoiceFilter new a: frac b: 1.0 - frac v: each steady]! !

!VoiceInterpolator methodsFor: 'initialization' stamp: 'len 5/4/98 02:06'!
initializeSlopes
	parametersStart _ current slopesWith: left.
	parametersEnd _ current slopesWith: right! !


!VoiceInterpolator methodsFor: 'accessing' stamp: 'len 5/4/98 00:07'!
currentFrames
	| top answer index params time frame |
	top _ 1.1 * f0hz.
	time _ 0.
	answer _ OrderedCollection new: current duration.
	current duration timesRepeat: [
		index _ 0.
		params _ current parameters collect: [ :each |
			index _ index + 1.
			(filters at: index) value: (self interpolate: (parametersStart at: index) with: (parametersEnd at: index) mid: each steady time: time duration: current duration)].
		frame _ current frameFromParameters: params.
		frame
			f0hz10: top.
			"b1phz: 60.0; b1hz: 60.0;
			b2phz: 90.0; b2hz: 90.0;
			b3phz: 150.0; b3hz: 150.0;"
			"fnphz: frame fnzhz."
		answer add: frame.
		top _ top - 0.5. "declination of f0 envelope 0.25 Hz / cS"
		time _ time + 1].
	^ answer! !

!VoiceInterpolator methodsFor: 'accessing' stamp: 'len 5/4/98 00:01'!
currentFrames2
	| tstress ntstress top answer index params time frame |
	top _ 1.1 * f0hz.
	time _ 0.
	answer _ OrderedCollection new: current duration.
	current duration timesRepeat: [
		"tstress _ tstress + 1.
		base _ top * 0.8. ""* 3 / 5""
		tstress = ntstress ifTrue: [
			tstress _ 0.
			ntstress _ current duration.
			stressStart _ stressEnd.
			...]"
		index _ 0.
		params _ current parameters collect: [ :each |
			index _ index + 1.
			(filters at: index) value: (self interpolate: (parametersStart at: index) with: (parametersEnd at: index) mid: each steady time: time duration: current duration)].
		frame _ current frameFromParameters: params.
		"Transcript cr; show: params printString."
		frame
			f0hz10: top;
			"b1phz: 60.0; b1hz: 60.0;
			b2phz: 90.0; b2hz: 90.0;
			b3phz: 150.0; b3hz: 150.0;"
			fnphz: left fn steady.
		answer add: frame.
		top _ top - 0.5. "declination of f0 envelope 0.25 Hz / cS"
		time _ time + 1].
	^ answer! !

!VoiceInterpolator methodsFor: 'accessing' stamp: 'len 5/5/98 00:56'!
framesFromVoiceElements: aCollection
	| answer |
	answer _ OrderedCollection new.
	left _ nil.
	current _ VoiceElement space.
	right _ current.
	aCollection do: [ :each |
		self left: current current: right right: each.
		answer addAll: self currentFrames].
	^ answer! !

!VoiceInterpolator methodsFor: 'accessing' stamp: 'len 5/4/98 02:10'!
left: aVoiceElement current: bVoiceElement right: cVoiceElement
	| needsFiltersInitialization |
	needsFiltersInitialization _ left isNil.
	left _ aVoiceElement.
	current _ bVoiceElement.
	right _ cVoiceElement.
	self initializeSlopes.
	needsFiltersInitialization ifTrue: [self initializeFilters]! !


!VoiceInterpolator methodsFor: 'private' stamp: 'len 5/4/98 02:45'!
interpolate: slope1 with: slope2 mid: mid time: time duration: duration
	| steady p1 p2 |
	steady _ duration - (slope1 x + slope2 x).
	steady < 0 "steady state cannot be reached"
		ifTrue: [p1 _ self linear: slope1 y with: mid time: time duration: slope1 x.
				p2 _ self linear: slope2 y with: mid time: duration - time duration: slope2 x.
				^ p2 - p1 * time / duration + p1].
	time < slope1 x
		ifTrue: [^ self linear: slope1 y with: mid time: time duration: slope1 x].
	^ time - slope1 x <= steady "steady state reached"
		ifTrue: [mid]
		ifFalse: [self linear: mid with: slope2 y time: time - slope1 x - steady duration: slope2 x]! !

!VoiceInterpolator methodsFor: 'private' stamp: 'len 4/11/98 18:24'!
linear: a with: b time: time duration: duration
	time <= 0 ifTrue: [^ a].
	time >= duration ifTrue: [^ b].
	^ b - a * time / duration + a! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

VoiceInterpolator class
	instanceVariableNames: ''!

!VoiceInterpolator class methodsFor: 'instance creation' stamp: 'len 4/12/98 19:51'!
new
	^ super new initialize! !


Object subclass: #VoiceSynthesizer
	instanceVariableNames: 'rnpp rps rcs rnpc rnz rgl rlp rout currentFrame f0hz10 timeCount t0 nper onemd nopen nmod avdb kskew skew pulseShape1 pulseShape2 vwave ampVoice parAmpVoice ampBreth ampFrica ampAspir ampBypas ampGain0 decay glotlast seed vlast nlast '
	classVariableNames: 'AllParallel DbToLinearTable F0Flutter NaturalSamples NFCascade SamplesPerFrame SamplingRate '
	poolDictionaries: ''
	category: 'Speech Synthesis'!
!VoiceSynthesizer commentStamp: 'len 5/5/98 01:10' prior: 0!
My instances are Klatt Cascade/Parallel Formant Synthesizers, first described by Dennis Klatt in [1]. An updated version of the model was described in [2].

This is an ugly implementation based upon Klatt3.03 and RSynth 2.0 (both are more or less the same, since the last one is a modification of the former). These two programs are available with C sources from ftp://svr-ftp.eng.cam.ac.uk/pub/comp.speech/synthesis/.

References
	[1] Klatt,D.H. Software for a cascade/parallel formant synthesizer, in the Journal of the Acoustic Society of America, pages 971-995, volume 67, number 3, March 1980.
	[2] Klatt,D.H. and Klatt, L.C. Analysis, synthesis and perception of voice quality variations among female and male talkers. In the Journal of the Acoustical Society of America, pages 820-857, volume 87, number 2. February 1990.
!


!VoiceSynthesizer methodsFor: 'initialization' stamp: 'len 5/4/98 01:12'!
initialize
	| flphz blphz |
	self initializeResonators.
	flphz _ 950 * SamplingRate / 10000.
	blphz _ 630 * SamplingRate / 10000.
	rlp frequency: flphz bandwidth: blphz.
	nper _ 0.
	t0 _ 0.
	nopen _ 0.
	nmod _ 0.
	timeCount _ 0.
	skew _ 0.
	seed _ 17.
	vlast _ 0.0.
	glotlast _ 0.0.
	nlast _ 0.! !

!VoiceSynthesizer methodsFor: 'initialization' stamp: 'len 4/9/98 22:40'!
initializeResonators
	| r1p r2p r3p r4p r5p r6p r1c r2c r3c r4c r5c r6c r7c r8c |
	rnpp _ DigitalResonator new name: 'parallel nasal pole resonator'.
	r1p _ DigitalResonator new name: 'parallel 1st formant resonator'.
	r2p _ DigitalResonator new name: 'parallel 2nd formant resonator'.
	r3p _ DigitalResonator new name: 'parallel 3rd formant resonator'.
	r4p _ DigitalResonator new name: 'parallel 4th formant resonator'.
	r5p _ DigitalResonator new name: 'parallel 5th formant resonator'.
	r6p _ DigitalResonator new name: 'parallel 6th formant resonator'.
	rps _ Array new: 6.
	rps at: 1 put: r1p; at: 2 put: r2p; at: 3 put: r3p;
		at: 4 put: r4p; at: 5 put: r5p; at: 6 put: r6p.
	r1c _ DigitalResonator new name: 'cascade 1st formant resonator'.
	r2c _ DigitalResonator new name: 'cascade 2nd formant resonator'.
	r3c _ DigitalResonator new name: 'cascade 3rd formant resonator'.
	r4c _ DigitalResonator new name: 'cascade 4th formant resonator'.
	r5c _ DigitalResonator new name: 'cascade 5th formant resonator'.
	r6c _ DigitalResonator new name: 'cascade 6th formant resonator'.
	r7c _ DigitalResonator new name: 'cascade 7th formant resonator'.
	r8c _ DigitalResonator new name: 'cascade 8th formant resonator'.
	rcs _ Array new: 8.
	rcs at: 1 put: r1c; at: 2 put: r2c; at: 3 put: r3c;
		at: 4 put: r4c; at: 5 put: r5c; at: 6 put: r6c;
		at: 7 put: r7c; at: 8 put: r8c.
	rnpc _ DigitalResonator new name: 'cascade nasal pole resonator'.
	rnz _ DigitalAntiResonator new name: 'cascade nasal zero anti-resonator'.
	rgl _ DigitalResonator new name: 'critic-damped glot low-pass filter'.
	rlp _ DigitalResonator new name: 'downsampling low-pass filter'.
	rout _ DigitalResonator new name: 'output low-pass filter'! !


!VoiceSynthesizer methodsFor: 'all' stamp: 'len 4/8/98 03:10'!
addFlutter
	"Add F0 flutter, as specified in:
		'Analysis, synthesis and perception of voice quality variations among
		female and male talkers' D.H. Klatt and L.C. Klatt JASA 87(2) February 1990.
	Flutter is added by applying a quasi-random element constructed from three
	slowly varying sine waves."

	| frame originalF0 fla flb flc fld fle deltaF0 |
	frame _ currentFrame.
	originalF0 _ frame f0hz10 / 10.
	fla _ F0Flutter / 50.
	flb _ originalF0 / 100.
	flc _ (2.0 * Float pi * 12.7 * timeCount) sin.
	fld _ (2.0 * Float pi * 7.1 * timeCount) sin.
	fle _ (2.0 * Float pi * 4.7 * timeCount) sin.
	deltaF0 _ fla * flb * (flc + fld + fle) * 10.
	f0hz10 _ f0hz10 + deltaF0.
	timeCount _ timeCount + 1! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 5/4/98 03:10'!
currentFrame: frame
	|  ampParF1 ampParF2 ampParF3 ampParF4 ampParF5 ampParF6 ampParFN gain0 |

	"self initialize."
	currentFrame _ frame.
	f0hz10 _ frame f0hz10.
	avdb _ frame avdb - 7 max: 0.
	kskew _ frame kskew.

	ampAspir _ (self dbToLinear: frame asp) * 0.05.
	ampFrica _ (self dbToLinear: frame af) * 0.25.
	parAmpVoice _ self dbToLinear: frame avpdb.

	"Fudge factors..."
	ampParF1 _ (self dbToLinear: frame a1) * 0.4.		"-7.96 dB"
	ampParF2 _ (self dbToLinear: frame a2) * 0.15.		"-16.5 dB"
	ampParF3 _ (self dbToLinear: frame a3) * 0.06.		"-24.4 dB"
	ampParF4 _ (self dbToLinear: frame a4) * 0.04.		"-28.0 dB"
	ampParF5 _ (self dbToLinear: frame a5) * 0.022.		"-33.2 dB"
	ampParF6 _ (self dbToLinear: frame a6) * 0.03.		"-30.5 dB"
	ampParFN _ (self dbToLinear: frame anp) * 0.6.	"-4.44 dB"
	ampBypas _ (self dbToLinear: frame ab) * 0.05.		"-26.0 dB"

	NFCascade >= 8
		ifTrue: [SamplingRate >= 16000	"Inside Nyquist rate?"
					ifTrue: [(rcs at: 8) frequency: 7500 bandwidth: 600]
					ifFalse: [NFCascade _ 6]].
	NFCascade >= 7
		ifTrue: [SamplingRate >= 16000	"Inside Nyquist rate?"
					ifTrue: [(rcs at: 7) frequency: 6500 bandwidth: 500]
					ifFalse: [NFCascade _ 6]].

	"Set coefficients of variable cascade resonators"
	NFCascade >= 6 ifTrue: [(rcs at: 6) frequency: frame f6hz bandwidth: frame b6hz].
	NFCascade >= 5 ifTrue: [(rcs at: 5) frequency: frame f5hz bandwidth: frame b5hz].
	(rcs at: 4) frequency: frame f4hz bandwidth: frame b4hz.
	(rcs at: 3) frequency: frame f3hz bandwidth: frame b3hz.
	(rcs at: 2) frequency: frame f2hz bandwidth: frame b2hz.
	(rcs at: 1) frequency: frame f1hz bandwidth: frame b1hz.

	"Set coefficients of nasal resonator and zero antiresonator"
	rnpc frequency: frame fnphz bandwidth: frame bnphz.
	rnz frequency: frame fnzhz bandwidth: frame bnzhz.

	"Set coefficients of parallel resonators, and amplitude of outputs"
	rnpp frequency: frame fnphz bandwidth: frame bnphz gain: ampParFN.
	(rps at: 1) frequency: frame f1hz bandwidth: frame b1phz gain: ampParF1.
	(rps at: 2) frequency: frame f2hz bandwidth: frame b2phz gain: ampParF2.
	(rps at: 3) frequency: frame f3hz bandwidth: frame b3phz gain: ampParF3.
	(rps at: 4) frequency: frame f4hz bandwidth: frame b4phz gain: ampParF4.
	(rps at: 5) frequency: frame f5hz bandwidth: frame b5phz gain: ampParF5.
	(rps at: 6) frequency: frame f6hz bandwidth: frame b6phz gain: ampParF6.

	"Flod overall gain into output resonator"
	gain0 _ frame gain0 - 3.
	gain0 <= 0 ifTrue: [gain0 _ 57].
	ampGain0 _ self dbToLinear: gain0.

	"Output low-pass filter"
	rout frequency: 0 bandwidth: SamplingRate gain: ampGain0! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 4/30/98 23:52'!
impulsiveSource
	vwave _ nper < 3
		ifTrue: [#(0.0 13000000.0 -13000000.0) at: nper + 1]
		ifFalse: [0.0].
	"Low-pass filter the differenciated impulse with a critically-damped
	second-order filter, time constant proportional to kopen."
	^ rgl value: vwave! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 5/4/98 01:26'!
naturalSource
	^ nper < nopen "see if glottis open"
		ifTrue: [pulseShape1 _ pulseShape1 - pulseShape2.
				vwave _ vwave + pulseShape1.
				vwave * 0.028]
		ifFalse: [vwave _ 0.0]! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 5/2/98 19:03'!
pitchSynchParReset: ns
	| b0 frame temp |
	frame _ currentFrame.
	b0 _ #(1200 1142 1088 1038 991 948 907 869 833 799 768 738 710 683 658 634 612 590 570 551 533 515 499 483 468 454 440 427 415 403 391 380 370 360 350 341 332 323 315 307 300 292 285 278 272 265 259 253 247 242 237 231 226 221 217 212 208 204 199 195 192 188 184 180 177 174 170 167 164 161 158 155 153 150 147 145 142 140 137 135 133 131 128 126 124 122 120 119 117 115 113 111 110 108 106 105 103 102 100 99 97 96 95 93 92 91 90 88 87 86 85 84 83 82 80 79 78 77 76 75 75 74 73 72 71 70 69 68 68 67 66 65 64 64 63 62 61 61 60 59 59 58 57 57 56 56 55 54 54 53 53 52 52 51 51 50 49 49 48 48 48 47 47 46 46 45 45 44 44 43 43 43 42 42 41 41 41 40 40 40 39 39 38 38 38 37 37 37 36 36 36 35 35 35 35 34 34 34 33 33 33 33 32 32 32 31 31 31 31 30 30 30 30 29 29 29 29 29 28 28 28 28 27 27).
	f0hz10 > 0
		ifTrue: ["t0 is 4 times the number of samples in one pitch period"
				t0 _ 40 * SamplingRate // f0hz10.
				ampVoice _ (self dbToLinear: avdb).

				"Duration of period before amplitude modulation"
				nmod _ t0.
				avdb > 0 ifTrue: [nmod _ nmod // 2].

				"Breathiness of voicing waveform"
				ampBreth _ (self dbToLinear: frame aturb) * 0.1.

				"Set open phase of glottal period where 40 <= open phase <= 263"
				nopen _ 4 * frame kopen.

				(self usingImpulsiveSource and: [nopen > 263]) ifTrue: [nopen _ 263].

				nopen >= (t0 - 1) ifTrue: [nopen _ t0 - 2].
				nopen < 40 ifTrue: [nopen _ 40].		"F0 max = 1000 Hz"

				"Reset pulse shapes 1 & 2, which determine
				shape of 'natural' glottal waveform."
				pulseShape2 _ (b0 at: nopen - 40 + 1) asFloat.
				pulseShape1 _ pulseShape2 * nopen * 0.333.

				"Reset width of 'impulsive' glottal pulse and make
				gain at F1 about constant"
				rgl frequency: 0 bandwidth: SamplingRate asFloat / nopen asFloat
					gain: (nopen * 0.00833) squared.

				"Truncate skewness so as not to exceed duration of closed phase
				of glottal period."
				temp _ t0 - nopen.
				kskew > temp ifTrue: [kskew _ temp].
				skew _ skew >= 0 ifTrue: [kskew] ifFalse: [kskew negated].

				"Add skewness to closed portion of voicing period"
				t0 _ t0 + skew.
				skew _ skew negated]
		ifFalse: [t0 _ 4.
				ampVoice _ 0.0.
				nmod _ t0.
				ampBreth _ 0.0.
				pulseShape1 _ 0.0.
				pulseShape2 _ 0.0].

	"Reset these parameters pitch synchronously or at update rate if f0=0"
	(t0 ~= 4 or: [ns = 0]) ifTrue: [
		"Set one-pole low-pass filter that tilts glottal source"
		decay _ 0.033 * frame tltdb.
		onemd _ decay > 0.0 ifTrue: [1.0 - decay] ifFalse: [1.0]]! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 4/9/98 03:22'!
sampledSource
	| index diff current next temp m |
	^ t0 ~= 0
		ifTrue: [index _ (temp _ NaturalSamples size - 2 * nper) // t0 + 1.
				diff _ temp \\ t0.
				current _ NaturalSamples at: index.
				next _ NaturalSamples at: index + 1.
				m _ t0 - 1.
				(diff * next + (current * (m - diff) // m)) asFloat * 0.1]
		ifFalse: [0.0]! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 5/4/98 03:07'!
soundForFrame: frame
	| buffer noise voice frics aspiration glotout
	parGlotout sourc temp out cascNextIn
	index counter top
	r1p r2p r3p r4p r5p r6p |

	buffer _ SoundBuffer newStereoSampleCount: SamplesPerFrame.
	self currentFrame: frame.
	self addFlutter.
	r1p _ rps at: 1.
	r2p _ rps at: 2.
	r3p _ rps at: 3.
	r4p _ rps at: 4.
	r5p _ rps at: 5.
	r6p _ rps at: 6.
	noise _ nlast.
	index _ 1.
	top _ SamplesPerFrame * 2 - 1.
	[index <= top] whileTrue: [
		"Get low-passed random number for aspiration and frication noise"
		seed _ ((seed * 1309) + 13849) bitAnd: 65535.

		"Tilt down noise spectrum by soft low-pass filter having
		a pole near the origin in the z-plane."
		noise _ (seed - 32768) asFloat / 4.0 + (0.75 * nlast).
		nlast _ noise.

		"Amplitude modulate noise (reduce noise amplitude during second
		half of glottal period) if voicing  simultaneously present."
		nper > nmod ifTrue: [noise _ noise * 0.5].

		"Compute frictation noise"
		frics _ ampFrica * noise.

		"Compute voicing waveform. Run glottal source simulation
		at 4 times normal sample rate to minimize quantization noise
		in period of female voice."
		counter _ 1.
		[counter <= 4] whileTrue: [
			voice _ self naturalSource asFloat.

			"Reset period when 'nper' reaches t0"
			nper >= t0 ifTrue: [nper _ 0. self pitchSynchParReset: index - 1 // 2].

			"Low-pass filter voicing waveform before downsampling
			from SamplingRate*4 to SamplingRate."
			voice _ rlp value: voice.

			"Increment counter that keeps track of SamplingRate*4 samples per second"
			nper _ nper + 1.
			counter _ counter + 1].

		"Tilt spectrum of voicing source down by soft low-pass filtering,
		amount of tilt determined by tltdb."
		voice _ voice * onemd + (vlast * decay).
		vlast _ voice.

		"Add breathiness during glottal open phase. Amount of
		breathiness determined by parameter aturb. Use seed
		rather than noise because noise is low-passed."
		nper < nopen ifTrue: [voice _ voice + (ampBreth * (seed - 32768) asFloat / 4.0)].

		"Set amplitude of voicing."
		glotout _ ampVoice * voice.
		parGlotout _ parAmpVoice * voice.

		"Compute aspiration amplitude and add to voicing source."
		aspiration _ ampAspir * noise.
		glotout _ glotout + aspiration.
		parGlotout _ parGlotout + aspiration.

		out _ 0.0.
		"Cascade vocal tract, excited by laryngeal sources.
		Nasal antiresonator, then formants FNP, F5, F4, F3, F2, F1."
		AllParallel ifFalse: [
			cascNextIn _ rnz value: glotout.
			cascNextIn _ rnpc value: cascNextIn.
			"Do not use unless sample rate >= 16000"
			NFCascade >= 8 ifTrue: [cascNextIn _ (rcs at: 8) value: cascNextIn].
			"Do not use unless sample rate >= 16000"
			NFCascade >= 7 ifTrue: [cascNextIn _ (rcs at: 7) value: cascNextIn].
			"Do not use unless long vocal tract or sample rate increased"
			NFCascade >= 6 ifTrue: [cascNextIn _ (rcs at: 6) value: cascNextIn].
			NFCascade >= 5 ifTrue: [cascNextIn _ (rcs at: 5) value: cascNextIn].
			NFCascade >= 4 ifTrue: [cascNextIn _ (rcs at: 4) value: cascNextIn].
			NFCascade >= 3 ifTrue: [cascNextIn _ (rcs at: 3) value: cascNextIn].
			NFCascade >= 2 ifTrue: [cascNextIn _ (rcs at: 2) value: cascNextIn].
			NFCascade >= 1 ifTrue: [out _ (rcs at: 1) value: cascNextIn]].

		"Excite parallel F1 and FNP by voicing waveform"
		sourc _ parGlotout.	"Source is voicing plus aspiration."

		"Standard parallel vocal tract formants F6, F5, F4, F3, F2,
		outputs added with alternating sign. Sound sourc for other
		parallel resonators is frication plus first difference of
		voicing waveform."

		out _ out + (r1p value: sourc) + (rnpp value: sourc).

		sourc _ frics + parGlotout - glotlast.
		glotlast _ parGlotout.

		out _ (r2p value: sourc) - (r3p value: sourc) + (r4p value: sourc) - (r5p value: sourc) + (r6p value: sourc) - out.

		"Apply bypas and output low-pass filter"
		out _ ampBypas * sourc - out.
		out _ rout value: out.
		temp _ (out * ampGain0) truncated.
		temp < -32768 ifTrue: [temp _ -32768].
		temp > 32767 ifTrue: [temp _ 32767].
		buffer at: index put: temp.
		buffer at: index + 1 put: temp.
		index _ index + 2].
	^ SampledSound samples: buffer samplingRate: SamplingRate * 2! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 5/2/98 18:47'!
soundForFrames: aCollection
	| sound answer |
	aCollection do: [ :each |
		sound _ self soundForFrame: each.
		answer _ answer isNil ifTrue: [sound] ifFalse: [answer, sound]].
	^ answer! !

!VoiceSynthesizer methodsFor: 'all' stamp: 'len 5/2/98 19:04'!
usingImpulsiveSource
	^ false! !


!VoiceSynthesizer methodsFor: 'private' stamp: 'len 5/4/98 01:36'!
dbToLinear: aNumber
	| dB |
	dB _ aNumber rounded.
	dB < 0 ifTrue: [dB _ 0].
	dB >= 88 ifTrue: [dB _ 87].
	^ (DbToLinearTable at: dB + 1) * 0.001! !


!VoiceSynthesizer methodsFor: 'testings' stamp: 'len 5/2/98 18:54'!
naturalSource2
	^ nper < nopen "see if glottis open"
		ifTrue: [pulseShape1 _ pulseShape1 - pulseShape2.
				vwave _ vwave + pulseShape1.
				vwave // 36 "old: * 0.028"]
		ifFalse: [vwave _ 0]! !

!VoiceSynthesizer methodsFor: 'testings' stamp: 'len 5/4/98 01:10'!
soundForFrame2: frame
	| buffer noise voice frics aspiration glotout
	parGlotout sourc temp out cascNextIn
	index counter top
	r1p r2p r3p r4p r5p r6p |

	buffer _ SoundBuffer newStereoSampleCount: SamplesPerFrame.
	self currentFrame: frame.
	self addFlutter.
	r1p _ rps at: 1.
	r2p _ rps at: 2.
	r3p _ rps at: 3.
	r4p _ rps at: 4.
	r5p _ rps at: 5.
	r6p _ rps at: 6.
	seed _ 17.
	noise _ 0.
	nlast _ 0.
	vlast _ 0.0.
	glotlast _ 0.0.
	index _ 1.
	top _ SamplesPerFrame * 2 - 1.
	[index <= top] whileTrue: [
		"Get low-passed random number for aspiration and frication noise"
		seed _ ((seed * 1309) + 13849) bitAnd: 65535.
		"Tilt down noise spectrum by soft low-pass filter having
		a pole near the origin in the z-plane."
		noise _ (seed - 32768) asFloat + (0.75 * nlast).
		nlast _ noise.

		"Amplitude modulate noise (reduce noise amplitude during second
		half of glottal period) if voicing  simultaneously present."
		nper > nmod ifTrue: [noise _ noise * 0.5].

"""		noise _ nper > nmod
""			ifTrue: [seed - 32768 + (noise * 3) bitShift: -3]
""			ifFalse: [seed - 32768 + (noise * 3) bitShift: -2]."

		"Compute frictation noise"
		sourc _ frics _ ampFrica * noise.

		"Compute voicing waveform. Run glottal source simulation
		at 4 times normal sample rate to minimize quantization noise
		in period of female voice."
		counter _ 1.
		[counter <= 4] whileTrue: [
			voice _ self naturalSource asFloat.

			"Reset period when 'nper' reaches t0"
			nper >= t0 ifTrue: [nper _ 0. self pitchSynchParReset: index - 1 // 2].

			"Low-pass filter voicing waveform before downsampling
			from SamplingRate*4 to SamplingRate."
			voice _ rlp value: voice.

			"Increment counter that keeps track of SamplingRate*4 samples per second"
			nper _ nper + 1.
			counter _ counter + 1].

		"Tilt spectrum of voicing source down by soft low-pass filtering,
		amount of tilt determined by tltdb."
		voice _ voice * onemd + (vlast * decay).
		vlast _ voice.

		"Add breathiness during glottal open phase. Amount of
		breathiness determined by parameter aturb. Use seed
		rather than noise because noise is low-passed."
		nper < nopen ifTrue: [voice _ voice + (ampBreth * (seed - 32768))].

		"Set amplitude of voicing."
		glotout _ ampVoice * voice.
		parGlotout _ parAmpVoice * voice.

		"Compute aspiration amplitude and add to voicing source."
		aspiration _ ampAspir * noise.
		glotout _ glotout + aspiration.
		parGlotout _ glotout.

		out _ 0.0.
		"Cascade vocal tract, excited by laryngeal sources.
		Nasal antiresonator, then formants FNP, F5, F4, F3, F2, F1."
		AllParallel ifFalse: [
			cascNextIn _ rnz value: glotout.
			cascNextIn _ rnpc value: cascNextIn.
			"Do not use unless sample rate >= 16000"
			NFCascade >= 8 ifTrue: [cascNextIn _ (rcs at: 8) value: cascNextIn].
			"Do not use unless sample rate >= 16000"
			NFCascade >= 7 ifTrue: [cascNextIn _ (rcs at: 7) value: cascNextIn].
			"Do not use unless long vocal tract or sample rate increased"
			NFCascade >= 6 ifTrue: [cascNextIn _ (rcs at: 6) value: cascNextIn].
			NFCascade >= 5 ifTrue: [cascNextIn _ (rcs at: 5) value: cascNextIn].
			NFCascade >= 4 ifTrue: [cascNextIn _ (rcs at: 4) value: cascNextIn].
			NFCascade >= 3 ifTrue: [cascNextIn _ (rcs at: 3) value: cascNextIn].
			NFCascade >= 2 ifTrue: [cascNextIn _ (rcs at: 2) value: cascNextIn].
			NFCascade >= 1 ifTrue: [out _ (rcs at: 1) value: cascNextIn]]
		ifTrue: ["all parallel"
			"rsynth hack!!!!!!"
			parGlotout _ rnz value: parGlotout.
			parGlotout _ rnpc value: parGlotout.
			out _ r1p value: parGlotout.
			sourc _ sourc + (parGlotout - glotlast).
			glotlast _ parGlotout].

		"Excite parallel F1 and FNP by voicing waveform"
"""		sourc _ parGlotout."	"Source is voicing plus aspiration."

		"Standard parallel vocal tract formants F6, F5, F4, F3, F2,
		outputs added with alternating sign. Sound sourc for other
		parallel resonators is frication plus first difference of
		voicing waveform."

"""		out _ out + (r1p value: sourc) + (rnpp value: sourc).

""		sourc _ frics + parGlotout - glotlast.
		glotlast _ parGlotout."

		out _ (r2p value: sourc) - (r3p value: sourc) + (r4p value: sourc) - (r5p value: sourc) + (r6p value: sourc) - out.

		"Apply bypas and output low-pass filter"
		out _ ampBypas * sourc - out.
		out _ rout value: out.
		temp _ (out * ampGain0) truncated.
		temp < -32768 ifTrue: [temp _ -32768].
		temp > 32767 ifTrue: [temp _ 32767].
		buffer at: index put: temp.
		buffer at: index + 1 put: temp.
		index _ index + 2].
	^ SampledSound samples: buffer samplingRate: SamplingRate * 2! !

!VoiceSynthesizer methodsFor: 'testings' stamp: 'len 5/4/98 01:22'!
soundForFrame: frame into: buffer startingAt: startIndex
	| noise voice frics aspiration glotout
	parGlotout sourc temp out cascNextIn
	index counter top
	r1p r2p r3p r4p r5p r6p |

	out _ 0.0. "Output of cascade branch, also final output."

	self currentFrame: frame.
	self addFlutter.
	r1p _ rps at: 1.
	r2p _ rps at: 2.
	r3p _ rps at: 3.
	r4p _ rps at: 4.
	r5p _ rps at: 5.
	r6p _ rps at: 6.
	noise _ 0.
	index _ 1.
	top _ SamplesPerFrame * 2.
	[index <= top] whileTrue: [
		"Get low-passed random number for aspiration and frication noise"
		seed _ ((seed * 1309) + 13849) bitAnd: 65535.
		noise _ (seed - 32768) asFloat / 4.0. "radom number between -8196.0 and 8196.0"

		"Tilt down noise spectrum by soft low-pass filter having
		a pole near the origin in the z-plane."
		noise _ noise + (0.75 * nlast).
		nlast _ noise.

		"Amplitude modulate noise (reduce noise amplitude during second
		half of glottal period) if voicing  simultaneously present."
		nper > nmod ifTrue: [noise _ noise * 0.5].

		"Compute frictation noise"
		sourc _ frics _ ampFrica * noise.

		"Compute voicing waveform. Run glottal source simulation
		at 4 times normal sample rate to minimize quantization noise
		in period of female voice."
		counter _ 1.
		[counter <= 4] whileTrue: [
			voice _ self naturalSource asFloat.

			"Reset period when 'nper' reaches t0"
			nper >= t0 ifTrue: [nper _ 0. self pitchSynchParReset: index - 1 // 2].

			"Low-pass filter voicing waveform before downsampling
			from SamplingRate*4 to SamplingRate."
			voice _ rlp value: voice.

			"Increment counter that keeps track of SamplingRate*4 samples per second"
			nper _ nper + 1.
			counter _ counter + 1].

		"Tilt spectrum of voicing source down by soft low-pass filtering,
		amount of tilt determined by tltdb."
		voice _ voice * onemd + (vlast * decay).
		vlast _ voice.

		"Add breathiness during glottal open phase. Amount of
		breathiness determined by parameter aturb. Use seed
		rather than noise because noise is low-passed."
		nper < nopen ifTrue: [voice _ voice + (ampBreth * (seed - 32768) asFloat / 4.0)].

		"Set amplitude of voicing."
		glotout _ ampVoice * voice.

		"Compute aspiration amplitude and add to voicing source."
		aspiration _ ampAspir * noise.
		glotout _ glotout + aspiration.

		parGlotout _ glotout.

		out _ 0.0.
		"Cascade vocal tract, excited by laryngeal sources.
		Nasal antiresonator, then formants FNP, F5, F4, F3, F2, F1."
		AllParallel ifFalse: [
			cascNextIn _ rnz value: glotout.
			cascNextIn _ rnpc value: cascNextIn.
			"Do not use unless sample rate >= 16000"
			NFCascade >= 8 ifTrue: [cascNextIn _ (rcs at: 8) value: cascNextIn].
			"Do not use unless sample rate >= 16000"
			NFCascade >= 7 ifTrue: [cascNextIn _ (rcs at: 7) value: cascNextIn].
			"Do not use unless long vocal tract or sample rate increased"
			NFCascade >= 6 ifTrue: [cascNextIn _ (rcs at: 6) value: cascNextIn].
			NFCascade >= 5 ifTrue: [cascNextIn _ (rcs at: 5) value: cascNextIn].
			NFCascade >= 4 ifTrue: [cascNextIn _ (rcs at: 4) value: cascNextIn].
			NFCascade >= 3 ifTrue: [cascNextIn _ (rcs at: 3) value: cascNextIn].
			NFCascade >= 2 ifTrue: [cascNextIn _ (rcs at: 2) value: cascNextIn].
			NFCascade >= 1 ifTrue: [out _ (rcs at: 1) value: cascNextIn].
			"Excite parallel F1 and FNP by voicing waveform.
			Source is voicing plus aspiration. Add in phase, boost lows for nasalized."
			"out _ out + (rnpp value: parGlotout) + (r1p value: parGlotout)"]
		ifTrue: ["is AllParallel"
			"rsynth hack"
			parGlotout _ rnz value: parGlotout.
			parGlotout _ rnpc value: parGlotout.
			"And just use r1p, not rnpp."
			out _ r1p value: parGlotout.
			"Second sourc for other parallel resonators is frication
			plus first difference of voicing waveform."
			sourc _ sourc + (parGlotout - glotlast).
			glotlast _ parGlotout].

		"Standard parallel vocal tract formants F6, F5, F4, F3, F2,
		outputs added with alternating sign. Sound sourc for other
		parallel resonators is frication plus first difference of
		voicing waveform."
		out _ (r6p value: sourc) - out.
		out _ (r5p value: sourc) - out.
		out _ (r4p value: sourc) - out.
		out _ (r3p value: sourc) - out.
		out _ (r2p value: sourc) - out.

		"Apply bypas and output low-pass filter"
		out _ ampBypas * sourc - out.
		out _ rout value: out.
		temp _ (out * ampGain0) rounded.
		temp < -32768 ifTrue: [temp _ -32768].
		temp > 32767 ifTrue: [temp _ 32767].
		buffer at: startIndex + index - 1 put: temp.
		buffer at: startIndex + index put: temp.
		index _ index + 2]! !

!VoiceSynthesizer methodsFor: 'testings' stamp: 'len 5/5/98 01:08'!
soundForFrames2: aCollection
	| sound answer |
	aCollection do: [ :each |
		sound _ self soundForFrame2: each.
		answer _ answer isNil ifTrue: [sound] ifFalse: [answer, sound]].
	^ answer! !

!VoiceSynthesizer methodsFor: 'testings' stamp: 'len 5/4/98 01:21'!
soundForFrames3: aCollection
	| buffer index |
	buffer _ SoundBuffer newStereoSampleCount: aCollection size * SamplesPerFrame.
	index _ 1.
	aCollection do: [ :each |
		self soundForFrame: each into: buffer startingAt: index.
		index _ SamplesPerFrame * 2 + index].
	^ SampledSound samples: buffer samplingRate: SamplingRate * 2! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

VoiceSynthesizer class
	instanceVariableNames: ''!

!VoiceSynthesizer class methodsFor: 'class initialization' stamp: 'len 5/4/98 00:52'!
initialize
	"VoiceSynthesizer initialize."

	| millisecondsPerFrame |
	millisecondsPerFrame _ 10.
	SamplingRate _ 8000.
	SamplesPerFrame _ (SamplingRate * millisecondsPerFrame / 1000) rounded.
	"When this is true, only the parallel branch is used:"
	AllParallel _ true.
	"Number of formants in the cascade branch:"
	NFCascade _ 0.
	"Percentage of F0 flutter to be applied to the synthesized speech:"
	F0Flutter _ 0.
	self initializeDbToLinearTable.
	self initializeNaturalSamples! !

!VoiceSynthesizer class methodsFor: 'class initialization' stamp: 'len 4/13/98 03:30'!
initializeDbToLinearTable
	"VoiceSynthesizer initializeDbToLinearTable."
	DbToLinearTable _
		#(0.0 0.0 0.0 0.0 0.0
		0.0 0.0 0.0 0.0 0.0
		0.0 0.0 0.0 6.0 7.0
		8.0 9.0 10.0 11.0 13.0
		14.0 16.0 18.0 20.0 22.0
		25.0 28.0 32.0 35.0 40.0
		45.0 51.0 57.0 64.0 71.0
		80.0 90.0 101.0 114.0 128.0
		142.0 159.0 179.0 202.0 227.0
		256.0 284.0 318.0 359.0 405.0
		455.0 512.0 568.0 638.0 719.0
		811.0 911.0 1024.0 1137.0 1276.0
		1438.0 1622.0 1823.0 2048.0 2273.0
		2552.0 2875.0 3244.0 3645.0 4096.0
		4547.0 5104 5751.0 6488.0 7291.0
		8192.0 9093.0 10207.0 11502.0 12976.0
		14582.0 16384.0 18350.0 20644.0 23429.0
		26214.0 29491.0 32767.0)! !

!VoiceSynthesizer class methodsFor: 'class initialization' stamp: 'len 4/13/98 03:30'!
initializeNaturalSamples
	"VoiceSynthesizer initializeNaturalSamples."

	NaturalSamples _
		#(-310 -400 530 356 224 89 23 -10 -58 -16 461 599 536 701 770
		605 497 461 560 404 110 224 131 104 -97 155 278 -154 -1165
		-598 737 125 -592 41 11 -247 -10 65 92 80 -304 71 167 -1 122
		233 161 -43 278 479 485 407 266 650 134 80 236 68 260 269 179
		53 140 275 293 296 104 257 152 311 182 263 245 125 314 140 44
		203 230 -235 -286 23 107 92 -91 38 464 443 176 98 -784 -2449
		-1891 -1045 -1600 -1462 -1384 -1261 -949 -730)! !


!VoiceSynthesizer class methodsFor: 'instance creation' stamp: 'len 4/8/98 03:33'!
new
	^ super new initialize! !


!VoiceSynthesizer class methodsFor: 'constants' stamp: 'len 4/8/98 03:34'!
samplingRate
	^ SamplingRate! !


!VoiceSynthesizer class methodsFor: 'examples' stamp: 'len 4/13/98 19:29'!
default
	^ self new! !


VoiceSynthesizer initialize!
